from __future__ import annotations

import json

from ..helper import filter_none
from ..base_provider import AsyncGeneratorProvider, ProviderModelMixin, FinishReason
from ...typing import Union, Optional, AsyncResult, Messages, ImageType
from ...requests import StreamSession, raise_for_status
from ...errors import MissingAuthError, ResponseError
from ...image import to_data_uri

class Openai(AsyncGeneratorProvider, ProviderModelMixin):
    label = "OpenAI API"
    url = "https://openai.com"
    working = True
    needs_auth = True
    supports_message_history = True
    supports_system_message = True
    default_model = ""

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        timeout: int = 120,
        image: ImageType = None,
        api_key: str = None,
        api_base: str = "https://api.openai.com/v1",
        temperature: float = None,
        max_tokens: int = None,
        top_p: float = None,
        stop: Union[str, list[str]] = None,
        stream: bool = False,
        headers: dict = None,
        extra_data: dict = {},
        **kwargs
    ) -> AsyncResult:
        if cls.needs_auth and api_key is None:
            raise MissingAuthError('Add a "api_key"')
        if image is not None:
            if not model and hasattr(cls, "default_vision_model"):
                model = cls.default_vision_model
            messages[-1]["content"] = [
                {
                    "type": "image_url",
                    "image_url": {"url": to_data_uri(image)}
                },
                {
                    "type": "text",
                    "text": messages[-1]["content"]
                }
            ]
        async with StreamSession(
            proxies={"all": proxy},
            headers=cls.get_headers(stream, api_key, headers),
            timeout=timeout
        ) as session:
            data = filter_none(
                messages=messages,
                model=cls.get_model(model),
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                stop=stop,
                stream=stream,
                **extra_data
            )
            async with session.post(f"{api_base.rstrip('/')}/chat/completions", json=data) as response:
                await raise_for_status(response)
                if not stream:
                    data = await response.json()
                    cls.raise_error(data)
                    choice = data["choices"][0]
                    if "content" in choice["message"]:
                        yield choice["message"]["content"].strip()
                    finish = cls.read_finish_reason(choice)
                    if finish is not None:
                        yield finish
                else:
                    first = True
                    async for line in response.iter_lines():
                        if line.startswith(b"data: "):
                            chunk = line[6:]
                            if chunk == b"[DONE]":
                                break
                            data = json.loads(chunk)
                            cls.raise_error(data)
                            choice = data["choices"][0]
                            if "content" in choice["delta"] and choice["delta"]["content"]:
                                delta = choice["delta"]["content"]
                                if first:
                                    delta = delta.lstrip()
                                if delta:
                                    first = False
                                    yield delta
                            finish = cls.read_finish_reason(choice)
                            if finish is not None:
                                yield finish

    @staticmethod
    def read_finish_reason(choice: dict) -> Optional[FinishReason]:
        if "finish_reason" in choice and choice["finish_reason"] is not None:
            return FinishReason(choice["finish_reason"])

    @staticmethod
    def raise_error(data: dict):
        if "error_message" in data:
            raise ResponseError(data["error_message"])
        elif "error" in data:
            raise ResponseError(f'Error {data["error"]["code"]}: {data["error"]["message"]}')

    @classmethod
    def get_headers(cls, stream: bool, api_key: str = None, headers: dict = None) -> dict:
        return {
            "Accept": "text/event-stream" if stream else "application/json",
            "Content-Type": "application/json",
            **(
                {"Authorization": f"Bearer {api_key}"}
                if api_key is not None else {}
            ),
            **({} if headers is None else headers)
        }



from __future__ import annotations

import json
import os
import os.path
import random
import logging

from ...requests import StreamSession, raise_for_status
from ...cookies import get_cookies_dir
from ...errors import MissingRequirementsError
from ... import debug

logging.basicConfig(level=logging.ERROR)

class NoValidHarFileError(Exception):
    ...

class arkReq:
    def __init__(self, arkURL, arkHeaders, arkBody, arkCookies, userAgent):
        self.arkURL = arkURL
        self.arkHeaders = arkHeaders
        self.arkBody = arkBody
        self.arkCookies = arkCookies
        self.userAgent = userAgent

telemetry_url = "https://telemetry.stytch.com/submit"
public_token = "public-token-live-507a52ad-7e69-496b-aee0-1c9863c7c819"
chatArks: list = None

def readHAR():
    harPath = []
    chatArks = []
    for root, dirs, files in os.walk(get_cookies_dir()):
        for file in files:
            if file.endswith(".har"):
                harPath.append(os.path.join(root, file))
        if harPath:
            break
    if not harPath:
        raise NoValidHarFileError("No .har file found")
    for path in harPath:
        with open(path, 'rb') as file:
            try:
                harFile = json.load(file)
            except json.JSONDecodeError:
                # Error: not a HAR file!
                continue
            for v in harFile['log']['entries']:
                if v['request']['url'] == telemetry_url:
                    chatArks.append(parseHAREntry(v))
    if not chatArks:
        raise NoValidHarFileError("No telemetry in .har files found")
    return chatArks

def parseHAREntry(entry) -> arkReq:
    tmpArk = arkReq(
        arkURL=entry['request']['url'],
        arkHeaders={h['name'].lower(): h['value'] for h in entry['request']['headers'] if h['name'].lower() not in ['content-length', 'cookie'] and not h['name'].startswith(':')},
        arkBody=entry['request']['postData']['text'],
        arkCookies={c['name']: c['value'] for c in entry['request']['cookies']},
        userAgent=""
    )
    tmpArk.userAgent = tmpArk.arkHeaders.get('user-agent', '')
    return tmpArk

async def sendRequest(tmpArk: arkReq, proxy: str = None):
    async with StreamSession(headers=tmpArk.arkHeaders, cookies=tmpArk.arkCookies, proxy=proxy) as session:
        async with session.post(tmpArk.arkURL, data=tmpArk.arkBody) as response:
            await raise_for_status(response)
            return await response.text()

async def create_telemetry_id(proxy: str = None):
    global chatArks
    if chatArks is None:
        chatArks = readHAR()
    return await sendRequest(random.choice(chatArks), proxy)

async def get_telemetry_ids(proxy: str = None) -> list:
    try:
        return [await create_telemetry_id(proxy)]
    except NoValidHarFileError as e:
        if debug.logging:
            logging.error(e)

    try:
        from nodriver import start
    except ImportError:
        raise MissingRequirementsError('Add .har file from you.com or install "nodriver" package | pip install -U nodriver')
    if debug.logging:
        logging.error('Getting telemetry_id for you.com with nodriver')

    browser = page = None
    try:
        browser = await start(
            browser_args=None if proxy is None else [f"--proxy-server={proxy}"],
        )
        page = await browser.get("https://you.com")
        while not await page.evaluate('"GetTelemetryID" in this'):
            await page.sleep(1)
        async def get_telemetry_id():
            return await page.evaluate(
                f'this.GetTelemetryID("{public_token}", "{telemetry_url}");',
                await_promise=True
            )
        return [await get_telemetry_id()]
    finally:
        try:
            if page is not None:
                await page.close()
            if browser is not None:
                await browser.stop()
        except Exception as e:
            if debug.logging:
                logging.error(e)



from __future__ import annotations

from ..providers.types          import BaseProvider, ProviderType
from ..providers.retry_provider import RetryProvider, IterListProvider
from ..providers.base_provider  import AsyncProvider, AsyncGeneratorProvider
from ..providers.create_images  import CreateImagesProvider

from .deprecated      import *
from .not_working     import *
from .selenium        import *
from .needs_auth      import *

from .AI365VIP         import AI365VIP
from .Allyfy           import Allyfy
from .Aura             import Aura
from .Bing             import Bing
from .BingCreateImages import BingCreateImages
from .Blackbox         import Blackbox
from .ChatGot          import ChatGot
from .Chatgpt4o        import Chatgpt4o
from .Chatgpt4Online   import Chatgpt4Online
from .ChatgptFree      import ChatgptFree
from .Cohere           import Cohere
from .DDG              import DDG
from .DeepInfra        import DeepInfra
from .DeepInfraImage   import DeepInfraImage
from .Feedough         import Feedough
from .FlowGpt          import FlowGpt
from .FreeChatgpt      import FreeChatgpt
from .FreeGpt          import FreeGpt
from .FreeNetfly       import FreeNetfly
from .GeminiPro        import GeminiPro
from .GeminiProChat    import GeminiProChat
from .GigaChat         import GigaChat
from .GptTalkRu        import GptTalkRu
from .HuggingChat      import HuggingChat
from .HuggingFace      import HuggingFace
from .HuggingFace      import HuggingFace
from .Koala            import Koala
from .Liaobots         import Liaobots
from .LiteIcoding      import LiteIcoding
from .Llama            import Llama
from .Local            import Local
from .MagickPenAsk     import MagickPenAsk
from .MagickPenChat    import MagickPenChat
from .Marsyoo          import Marsyoo
from .MetaAI           import MetaAI
from .MetaAIAccount    import MetaAIAccount
from .Ollama           import Ollama
from .PerplexityLabs   import PerplexityLabs
from .Phind            import Phind
from .Pi               import Pi
from .Pizzagpt         import Pizzagpt
from .Reka             import Reka
from .Replicate        import Replicate
from .ReplicateHome    import ReplicateHome
from .Rocks            import Rocks
from .TeachAnything    import TeachAnything
from .Vercel           import Vercel
from .WhiteRabbitNeo   import WhiteRabbitNeo
from .You              import You

import sys

__modules__: list = [
    getattr(sys.modules[__name__], provider) for provider in dir()
    if not provider.startswith("__")
]
__providers__: list[ProviderType] = [
    provider for provider in __modules__
    if isinstance(provider, type)
    and issubclass(provider, BaseProvider)
]
__all__: list[str] = [
    provider.__name__ for provider in __providers__
]
__map__: dict[str, ProviderType] = dict([
    (provider.__name__, provider) for provider in __providers__
])

class ProviderUtils:
    convert: dict[str, ProviderType] = __map__



from __future__ import annotations

import secrets
import json
from aiohttp import ClientSession

from ...typing import AsyncResult, Messages
from ..base_provider import AsyncGeneratorProvider
from ..helper import format_prompt

class GptGod(AsyncGeneratorProvider):
    url = "https://gptgod.site"
    working = False
    supports_gpt_35_turbo = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        
        headers = {
            "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/118.0",
            "Accept": "text/event-stream",
            "Accept-Language": "de,en-US;q=0.7,en;q=0.3",
            "Accept-Encoding": "gzip, deflate, br",
            "Alt-Used": "gptgod.site",
            "Connection": "keep-alive",
            "Referer": f"{cls.url}/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "Pragma": "no-cache",
            "Cache-Control": "no-cache",
        }

        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "content": prompt,
                "id": secrets.token_hex(16).zfill(32)
            }
            async with session.get(f"{cls.url}/api/session/free/gpt3p5", params=data, proxy=proxy) as response:
                response.raise_for_status()
                event = None
                async for line in response.content:
                   # print(line)

                    if line.startswith(b'event: '):
                        event = line[7:-1]
                    
                    elif event == b"data" and line.startswith(b"data: "):
                        data = json.loads(line[6:-1])
                        if data:
                            yield data
                    
                    elif event == b"done":
                        break


from __future__ import annotations

import json, random, requests, threading
from aiohttp import ClientSession

from ..typing import CreateResult, Messages
from .base_provider import AbstractProvider
from .helper import format_prompt

class Cohere(AbstractProvider):
    url                   = "https://cohereforai-c4ai-command-r-plus.hf.space"
    working               = False
    supports_gpt_35_turbo = False
    supports_gpt_4        = False
    supports_stream       = True
    
    @staticmethod
    def create_completion(
        model: str,
        messages: Messages,
        stream: bool,
        proxy: str = None,
        max_retries: int = 6,
        **kwargs
    ) -> CreateResult:
        
        prompt = format_prompt(messages)
        
        headers = {
            'accept': 'text/event-stream',
            'accept-language': 'en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3',
            'cache-control': 'no-cache',
            'pragma': 'no-cache',
            'referer': 'https://cohereforai-c4ai-command-r-plus.hf.space/?__theme=light',
            'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
        }
        
        session_hash = ''.join(random.choices("abcdefghijklmnopqrstuvwxyz0123456789", k=11))

        params = {
            'fn_index': '1',
            'session_hash': session_hash,
        }

        response = requests.get(
            'https://cohereforai-c4ai-command-r-plus.hf.space/queue/join',
            params=params,
            headers=headers,
            stream=True
        )
        
        completion = ''

        for line in response.iter_lines():
            if line:
                json_data = json.loads(line[6:])
                
                if b"send_data" in (line):
                    event_id = json_data["event_id"]
                    
                    threading.Thread(target=send_data, args=[session_hash, event_id, prompt]).start()
                
                if b"process_generating" in line or b"process_completed" in line:
                    token = (json_data['output']['data'][0][0][1])
                    
                    yield (token.replace(completion, ""))
                    completion = token

def send_data(session_hash, event_id, prompt):
    headers = {
        'accept': '*/*',
        'accept-language': 'en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3',
        'cache-control': 'no-cache',
        'content-type': 'application/json',
        'origin': 'https://cohereforai-c4ai-command-r-plus.hf.space',
        'pragma': 'no-cache',
        'referer': 'https://cohereforai-c4ai-command-r-plus.hf.space/?__theme=light',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"macOS"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
    }

    json_data = {
        'data': [
            prompt,
            '',
            [],
        ],
        'event_data': None,
        'fn_index': 1,
        'session_hash': session_hash,
        'event_id': event_id
    }
    
    requests.post('https://cohereforai-c4ai-command-r-plus.hf.space/queue/data',
                    json = json_data, headers=headers)


from __future__ import annotations
from typing import Generator, Optional, Dict, Any, Union, List
import random
import asyncio
import base64

from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from ..typing import AsyncResult, Messages
from ..requests import StreamSession, raise_for_status
from ..errors import ResponseError
from ..image import ImageResponse

class ReplicateHome(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://replicate.com"
    parent = "Replicate"
    working = True
    default_model = 'stability-ai/stable-diffusion-3'
    models = [
		# Models for image generation
        'stability-ai/stable-diffusion-3',
        'bytedance/sdxl-lightning-4step',
        'playgroundai/playground-v2.5-1024px-aesthetic',
        
        # Models for image generation
        'meta/meta-llama-3-70b-instruct',
        'mistralai/mixtral-8x7b-instruct-v0.1',
        'google-deepmind/gemma-2b-it',
    ]

    versions = {
		# Model versions for generating images
        'stability-ai/stable-diffusion-3': [
            "527d2a6296facb8e47ba1eaf17f142c240c19a30894f437feee9b91cc29d8e4f"
        ],
        'bytedance/sdxl-lightning-4step': [
            "5f24084160c9089501c1b3545d9be3c27883ae2239b6f412990e82d4a6210f8f"
        ],
        'playgroundai/playground-v2.5-1024px-aesthetic': [
            "a45f82a1382bed5c7aeb861dac7c7d191b0fdf74d8d57c4a0e6ed7d4d0bf7d24"
        ],
        
        
        # Model versions for text generation
        'meta/meta-llama-3-70b-instruct': [
            "dp-cf04fe09351e25db628e8b6181276547"
        ],
        'mistralai/mixtral-8x7b-instruct-v0.1': [
            "dp-89e00f489d498885048e94f9809fbc76"
        ],
        'google-deepmind/gemma-2b-it': [
            "dff94eaf770e1fc211e425a50b51baa8e4cac6c39ef074681f9e39d778773626"
        ]
    }

    image_models = {"stability-ai/stable-diffusion-3", "bytedance/sdxl-lightning-4step", "playgroundai/playground-v2.5-1024px-aesthetic"}
    text_models = {"meta/meta-llama-3-70b-instruct", "mistralai/mixtral-8x7b-instruct-v0.1", "google-deepmind/gemma-2b-it"}

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        **kwargs: Any
    ) -> Generator[Union[str, ImageResponse], None, None]:
        yield await cls.create_async(messages[-1]["content"], model, **kwargs)

    @classmethod
    async def create_async(
        cls,
        prompt: str,
        model: str,
        api_key: Optional[str] = None,
        proxy: Optional[str] = None,
        timeout: int = 180,
        version: Optional[str] = None,
        extra_data: Dict[str, Any] = {},
        **kwargs: Any
    ) -> Union[str, ImageResponse]:
        headers = {
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'en-US',
            'Connection': 'keep-alive',
            'Origin': cls.url,
            'Referer': f'{cls.url}/',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
            'sec-ch-ua': '"Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
        }

        if version is None:
            version = random.choice(cls.versions.get(model, []))
        if api_key is not None:
            headers["Authorization"] = f"Bearer {api_key}"

        async with StreamSession(
            proxies={"all": proxy},
            headers=headers,
            timeout=timeout
        ) as session:
            data = {
                "input": {
                    "prompt": prompt,
                    **extra_data
                },
                "version": version
            }
            if api_key is None:
                data["model"] = cls.get_model(model)
                url = "https://homepage.replicate.com/api/prediction"
            else:
                url = "https://api.replicate.com/v1/predictions"
            async with session.post(url, json=data) as response:
                await raise_for_status(response)
                result = await response.json()
            if "id" not in result:
                raise ResponseError(f"Invalid response: {result}")

            while True:
                if api_key is None:
                    url = f"https://homepage.replicate.com/api/poll?id={result['id']}"
                else:
                    url = f"https://api.replicate.com/v1/predictions/{result['id']}"
                async with session.get(url) as response:
                    await raise_for_status(response)
                    result = await response.json()
                    if "status" not in result:
                        raise ResponseError(f"Invalid response: {result}")
                    if result["status"] == "succeeded":
                        output = result['output']
                        if model in cls.text_models:
                            return ''.join(output) if isinstance(output, list) else output
                        elif model in cls.image_models:
                            images: List[Any] = output
                            images = images[0] if len(images) == 1 else images
                            return ImageResponse(images, prompt)
                    elif result["status"] == "failed":
                        raise ResponseError(f"Prediction failed: {result}")
                    await asyncio.sleep(0.5)



from __future__ import annotations

import json
from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider
from .helper import format_prompt


class Chatgpt4Online(AsyncGeneratorProvider):
    url = "https://chatgpt4online.org"
    api_endpoint = "/wp-json/mwai-ui/v1/chats/submit"
    working = True
    supports_gpt_4 = True
    
    async def get_nonce():
        async with ClientSession() as session:
            async with session.post(f"https://chatgpt4online.org/wp-json/mwai/v1/start_session") as response:
                return (await response.json())["restNonce"]
    
    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "text/event-stream",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "dnt": "1",
            "origin": cls.url,
            "priority": "u=1, i",
            "referer": f"{cls.url}/",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "x-wp-nonce": await cls.get_nonce(),
        }

        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "botId": "default",
                "newMessage": prompt,
                "stream": True,
            }

            async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy) as response:
                response.raise_for_status()
                full_response = ""

                async for chunk in response.content.iter_any():
                    if chunk:
                        try:
                            # Extract the JSON object from the chunk
                            for line in chunk.decode().splitlines():
                                if line.startswith("data: "):
                                    json_data = json.loads(line[6:])
                                    if json_data["type"] == "live":
                                        full_response += json_data["data"]
                                    elif json_data["type"] == "end":
                                        final_data = json.loads(json_data["data"])
                                        full_response = final_data["reply"]
                                        break
                        except json.JSONDecodeError:
                            continue

                yield full_response





from .AItianhu        import AItianhu
from .Aichatos        import Aichatos
from .Bestim          import Bestim
from .ChatBase        import ChatBase
from .ChatForAi       import ChatForAi
from .ChatgptAi       import ChatgptAi
from .ChatgptDemo     import ChatgptDemo
from .ChatgptDemoAi   import ChatgptDemoAi
from .ChatgptLogin    import ChatgptLogin
from .ChatgptNext     import ChatgptNext
from .ChatgptX        import ChatgptX
from .Chatxyz         import Chatxyz
from .Cnote           import Cnote
from .Feedough        import Feedough
from .Gpt6            import Gpt6
from .GptChatly       import GptChatly
from .GptForLove      import GptForLove
from .GptGo           import GptGo
from .GptGod          import GptGod
from .OnlineGpt       import OnlineGpt



from __future__ import annotations

from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class AI365VIP(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://chat.ai365vip.com"
    api_endpoint = "/api/chat"
    working = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = True
    default_model = 'gpt-3.5-turbo'
    models = [
        'gpt-3.5-turbo',
        'gpt-4o',
        'claude-3-haiku-20240307',
    ]
    model_aliases = {
        "claude-3-haiku": "claude-3-haiku-20240307",
    }

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "dnt": "1",
            "origin": "https://chat.ai365vip.com",
            "priority": "u=1, i",
            "referer": "https://chat.ai365vip.com/en",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
        }
        async with ClientSession(headers=headers) as session:
            data = {
				"model": {
					"id": model,
					"name": {
						"gpt-3.5-turbo": "GPT-3.5",
						"claude-3-haiku-20240307": "claude-3-haiku",
						"gpt-4o": "GPT-4O"
					}.get(model, model),
				},
				"messages": [{"role": "user", "content": format_prompt(messages)}],
				"prompt": "You are a helpful assistant.",
			}
            async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for chunk in response.content:
                    if chunk:
                        yield chunk.decode()



from __future__ import annotations

import json
from typing import AsyncGenerator, Optional, List, Dict, Union, Any
from aiohttp import ClientSession, BaseConnector, ClientResponse

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider
from .helper import get_random_string, get_connector
from ..requests import raise_for_status

class Koala(AsyncGeneratorProvider):
    url = "https://koala.sh"
    working = True
    supports_gpt_35_turbo = True
    supports_message_history = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: Optional[str] = None,
        connector: Optional[BaseConnector] = None,
        **kwargs: Any
    ) -> AsyncGenerator[Dict[str, Union[str, int, float, List[Dict[str, Any]], None]], None]:
        if not model:
            model = "gpt-3.5-turbo"

        headers = {
            "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:122.0) Gecko/20100101 Firefox/122.0",
            "Accept": "text/event-stream",
            "Accept-Language": "de,en-US;q=0.7,en;q=0.3",
            "Accept-Encoding": "gzip, deflate, br",
            "Referer": f"{cls.url}/chat",
            "Flag-Real-Time-Data": "false",
            "Visitor-ID": get_random_string(20),
            "Origin": cls.url,
            "Alt-Used": "koala.sh",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "TE": "trailers",
        }

        async with ClientSession(headers=headers, connector=get_connector(connector, proxy)) as session:
            input_text = messages[-1]["content"]
            system_messages = " ".join(
                message["content"] for message in messages if message["role"] == "system"
            )
            if system_messages:
                input_text += f" {system_messages}"

            data = {
                "input": input_text,
                "inputHistory": [
                    message["content"]
                    for message in messages[:-1]
                    if message["role"] == "user"
                ],
                "outputHistory": [
                    message["content"]
                    for message in messages
                    if message["role"] == "assistant"
                ],
                "model": model,
            }

            async with session.post(f"{cls.url}/api/gpt/", json=data, proxy=proxy) as response:
                await raise_for_status(response)
                async for chunk in cls._parse_event_stream(response):
                    yield chunk

    @staticmethod
    async def _parse_event_stream(response: ClientResponse) -> AsyncGenerator[Dict[str, Any], None]:
        async for chunk in response.content:
            if chunk.startswith(b"data: "):
                yield json.loads(chunk[6:])



from __future__ import annotations
import json
from aiohttp import ClientSession
from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class FreeChatgpt(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://chat.chatgpt.org.uk"
    api_endpoint = "/api/openai/v1/chat/completions"
    working = True
    supports_gpt_35_turbo = True
    default_model = 'gpt-3.5-turbo'
    models = [
        'gpt-3.5-turbo',
        'SparkDesk-v1.1',
        'deepseek-coder',
        'deepseek-chat',
        'Qwen2-7B-Instruct',
        'glm4-9B-chat',
        'chatglm3-6B',
        'Yi-1.5-9B-Chat',
    ]

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "application/json, text/event-stream",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "dnt": "1",
            "origin": cls.url,
            "referer": f"{cls.url}/",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
        }
        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "messages": [
                    {"role": "system", "content": "\nYou are ChatGPT, a large language model trained by OpenAI.\nKnowledge cutoff: 2021-09\nCurrent model: gpt-3.5-turbo\nCurrent time: Thu Jul 04 2024 21:35:59 GMT+0300 (Eastern European Summer Time)\nLatex inline: \\(x^2\\) \nLatex block: $$e=mc^2$$\n\n"},
                    {"role": "user", "content": prompt}
                ],
                "stream": True,
                "model": model,
                "temperature": 0.5,
                "presence_penalty": 0,
                "frequency_penalty": 0,
                "top_p": 1
            }
            async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy) as response:
                response.raise_for_status()
                accumulated_text = ""
                async for line in response.content:
                    if line:
                        line_str = line.decode().strip()
                        if line_str == "data: [DONE]":
                            yield accumulated_text
                            break
                        elif line_str.startswith("data: "):
                            try:
                                chunk = json.loads(line_str[6:])
                                delta_content = chunk.get("choices", [{}])[0].get("delta", {}).get("content", "")
                                accumulated_text += delta_content
                            except json.JSONDecodeError:
                                pass



from __future__ import annotations

import json
from aiohttp import ClientSession

from ...typing import AsyncResult, Messages
from ..base_provider import AsyncGeneratorProvider

class ChatgptNext(AsyncGeneratorProvider):
    url = "https://www.chatgpt-free.cc"
    working = False
    supports_gpt_35_turbo = True
    supports_message_history = True
    supports_system_message = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        max_tokens: int = None,
        temperature: float = 0.7,
        top_p: float = 1,
        presence_penalty: float = 0,
        frequency_penalty: float = 0,
        **kwargs
    ) -> AsyncResult:
        if not model:
            model = "gpt-3.5-turbo"
        headers = {
            "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:122.0) Gecko/20100101 Firefox/122.0",
            "Accept": "text/event-stream",
            "Accept-Language": "de,en-US;q=0.7,en;q=0.3",
            "Accept-Encoding": "gzip, deflate, br",
            "Content-Type": "application/json",
            "Referer": "https://chat.fstha.com/",
            "x-requested-with": "XMLHttpRequest",
            "Origin": "https://chat.fstha.com",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "Authorization": "Bearer ak-chatgpt-nice",
            "Connection": "keep-alive",
            "Alt-Used": "chat.fstha.com",
        }
        async with ClientSession(headers=headers) as session:
            data = {
                "messages": messages,
                "stream": True,
                "model": model,
                "temperature": temperature,
                "presence_penalty": presence_penalty,
                "frequency_penalty": frequency_penalty,
                "top_p": top_p,
                "max_tokens": max_tokens,
            }
            async with session.post(f"https://chat.fstha.com/api/openai/v1/chat/completions", json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for chunk in response.content:
                    if chunk.startswith(b"data: [DONE]"):
                        break
                    if chunk.startswith(b"data: "):
                        content = json.loads(chunk[6:])["choices"][0]["delta"].get("content")
                        if content:
                            yield content



from __future__ import annotations

import requests
from ..typing import AsyncResult, Messages
from .needs_auth.Openai import Openai

class DeepInfra(Openai):
    label = "DeepInfra"
    url = "https://deepinfra.com"
    working = True
    needs_auth = False
    supports_stream = True
    supports_message_history = True
    default_model = "meta-llama/Meta-Llama-3-70B-Instruct"
    default_vision_model = "llava-hf/llava-1.5-7b-hf"
    model_aliases = {
        'dbrx-instruct': 'databricks/dbrx-instruct',
    }

    @classmethod
    def get_models(cls):
        if not cls.models:
            url = 'https://api.deepinfra.com/models/featured'
            models = requests.get(url).json()
            cls.models = [model['model_name'] for model in models if model["type"] == "text-generation"]
        return cls.models

    @classmethod
    def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        stream: bool,
        api_base: str = "https://api.deepinfra.com/v1/openai",
        temperature: float = 0.7,
        max_tokens: int = 1028,
        **kwargs
    ) -> AsyncResult:
        headers = {
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'en-US',
            'Connection': 'keep-alive',
            'Origin': 'https://deepinfra.com',
            'Referer': 'https://deepinfra.com/',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
            'X-Deepinfra-Source': 'web-embed',
            'sec-ch-ua': '"Google Chrome";v="119", "Chromium";v="119", "Not?A_Brand";v="24"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
        }
        return super().create_async_generator(
            model, messages,
            stream=stream,
            api_base=api_base,
            temperature=temperature,
            max_tokens=max_tokens,
            headers=headers,
            **kwargs
        )


from __future__ import annotations

import json
from aiohttp import ClientSession

from ...typing import AsyncResult, Messages
from ..base_provider import AsyncGeneratorProvider

class Chatxyz(AsyncGeneratorProvider):
    url = "https://chat.3211000.xyz"
    working = False
    supports_gpt_35_turbo = True
    supports_message_history = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            'Accept': 'text/event-stream',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'en-US,en;q=0.5',
            'Alt-Used': 'chat.3211000.xyz',
            'Content-Type': 'application/json',
            'Host': 'chat.3211000.xyz',
            'Origin': 'https://chat.3211000.xyz',
            'Referer': 'https://chat.3211000.xyz/',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
            'TE': 'trailers',
            'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:121.0) Gecko/20100101 Firefox/121.0',
            'x-requested-with': 'XMLHttpRequest'
        }   
        async with ClientSession(headers=headers) as session:
            data = {
                "messages": messages,
                "stream": True,
                "model": "gpt-3.5-turbo",
                "temperature": 0.5,
                "presence_penalty": 0,
                "frequency_penalty": 0,
                "top_p": 1,
                **kwargs
            }    
            async with session.post(f'{cls.url}/api/openai/v1/chat/completions', json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for chunk in response.content:
                    line = chunk.decode() 
                    if line.startswith("data: [DONE]"):
                            break
                    elif line.startswith("data: "):
                            line = json.loads(line[6:])
                            chunk = line["choices"][0]["delta"].get("content")
                            if(chunk):
                                yield chunk


from __future__ import annotations

import argparse

from g4f import Provider
from g4f.gui.run import gui_parser, run_gui_args

def main():
    parser = argparse.ArgumentParser(description="Run gpt4free")
    subparsers = parser.add_subparsers(dest="mode", help="Mode to run the g4f in.")
    api_parser = subparsers.add_parser("api")
    api_parser.add_argument("--bind", default="0.0.0.0:1337", help="The bind string.")
    api_parser.add_argument("--debug", action="store_true", help="Enable verbose logging.")
    api_parser.add_argument("--model", default=None, help="Default model for chat completion. (incompatible with --debug and --workers)")
    api_parser.add_argument("--provider", choices=[provider.__name__ for provider in Provider.__providers__ if provider.working],
                            default=None, help="Default provider for chat completion. (incompatible with --debug and --workers)")
    api_parser.add_argument("--proxy", default=None, help="Default used proxy.")
    api_parser.add_argument("--workers", type=int, default=None, help="Number of workers.")
    api_parser.add_argument("--disable-colors", action="store_true", help="Don't use colors.")
    api_parser.add_argument("--ignore-cookie-files", action="store_true", help="Don't read .har and cookie files.")
    api_parser.add_argument("--g4f-api-key", type=str, default=None, help="Sets an authentication key for your API. (incompatible with --debug and --workers)")
    api_parser.add_argument("--ignored-providers", nargs="+", choices=[provider.__name__ for provider in Provider.__providers__ if provider.working],
                            default=[], help="List of providers to ignore when processing request. (incompatible with --debug and --workers)")
    subparsers.add_parser("gui", parents=[gui_parser()], add_help=False)

    args = parser.parse_args()
    if args.mode == "api":
        run_api_args(args)
    elif args.mode == "gui":
        run_gui_args(args)
    else:
        parser.print_help()
        exit(1)

def run_api_args(args):
    from g4f.api import AppConfig, run_api

    AppConfig.set_config(
        ignore_cookie_files=args.ignore_cookie_files,
        ignored_providers=args.ignored_providers,
        g4f_api_key=args.g4f_api_key,
        defaults={
            "model": args.model,
            "provider": args.provider,
            "proxy": args.proxy
        }
    )
    run_api(
        bind=args.bind,
        debug=args.debug,
        workers=args.workers,
        use_colors=not args.disable_colors
    )

if __name__ == "__main__":
    main()



from __future__ import annotations

import uuid

import requests

from ...typing import Any, CreateResult
from ..base_provider import AbstractProvider


class V50(AbstractProvider):
    url                     = 'https://p5.v50.ltd'
    supports_gpt_35_turbo   = True
    supports_stream         = False
    needs_auth              = False
    working                 = False

    @staticmethod
    def create_completion(
        model: str,
        messages: list[dict[str, str]],
        stream: bool, **kwargs: Any) -> CreateResult:
        
        conversation = (
            "\n".join(
                f"{message['role']}: {message['content']}" for message in messages
            )
            + "\nassistant: "
        )
        payload = {
            "prompt"        : conversation,
            "options"       : {},
            "systemMessage" : ".",
            "temperature"   : kwargs.get("temperature", 0.4),
            "top_p"         : kwargs.get("top_p", 0.4),
            "model"         : model,
            "user"          : str(uuid.uuid4())
        }

        headers = {
            'authority'         : 'p5.v50.ltd',
            'accept'            : 'application/json, text/plain, */*',
            'accept-language'   : 'id-ID,id;q=0.9,en-US;q=0.8,en;q=0.7',
            'content-type'      : 'application/json',
            'origin'            : 'https://p5.v50.ltd',
            'referer'           : 'https://p5.v50.ltd/',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest'    : 'empty',
            'sec-fetch-mode'    : 'cors',
            'sec-fetch-site'    : 'same-origin',
            'user-agent'        : 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36'
        }
        response = requests.post(
            "https://p5.v50.ltd/api/chat-process",
            json=payload,
            headers=headers,
            proxies=kwargs.get('proxy', {}),
        )

        if "https://fk1.v50.ltd" not in response.text:
            yield response.text


from __future__ import annotations

from typing import Any, Dict

from aiohttp import ClientSession, ClientTimeout

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class TeachAnything(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://www.teach-anything.com"
    api_endpoint = "/api/generate"
    working = True
    default_model = "llama-3-70b-instruct"

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str | None = None,
        **kwargs: Any
    ) -> AsyncResult:
        headers = cls._get_headers()
        
        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {"prompt": prompt}
            
            timeout = ClientTimeout(total=60)
            
            async with session.post(
                f"{cls.url}{cls.api_endpoint}",
                json=data,
                proxy=proxy,
                timeout=timeout
            ) as response:
                response.raise_for_status()
                async for chunk in response.content.iter_any():
                    if chunk:
                        yield chunk.decode()

    @staticmethod
    def _get_headers() -> Dict[str, str]:
        return {
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "dnt": "1",
            "origin": "https://www.teach-anything.com",
            "priority": "u=1, i",
            "referer": "https://www.teach-anything.com/",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
        }



from __future__ import annotations

from ..typing import AsyncResult, Messages, Cookies
from .helper import format_prompt, get_cookies
from .MetaAI import MetaAI

class MetaAIAccount(MetaAI):
    needs_auth = True
    parent = "MetaAI"
    image_models = ["meta"]

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        cookies: Cookies = None,
        **kwargs
    ) -> AsyncResult:
        cookies = get_cookies(".meta.ai", True, True) if cookies is None else cookies
        async for chunk in cls(proxy).prompt(format_prompt(messages), cookies):
            yield chunk


from __future__ import annotations

import json
import aiohttp
import asyncio
from typing import Optional
import base64

from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import get_connector
from ..typing import AsyncResult, Messages
from ..requests.raise_for_status import raise_for_status
from ..providers.conversation import BaseConversation

class DDG(AsyncGeneratorProvider, ProviderModelMixin):
    url = base64.b64decode("aHR0cHM6Ly9kdWNrZHVja2dvLmNvbS9haWNoYXQ=").decode("utf-8")
    working = True
    supports_gpt_35_turbo = True
    supports_message_history = True

    default_model = "gpt-4o-mini"
    models = ["gpt-4o-mini", "claude-3-haiku-20240307", "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo", "mistralai/Mixtral-8x7B-Instruct-v0.1"]
    model_aliases = {
        "gpt-4": "gpt-4o-mini",
        "gpt-4o": "gpt-4o-mini",
        "claude-3-haiku": "claude-3-haiku-20240307",
        "llama-3.1-70b": "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
        "mixtral-8x7B": "mistralai/Mixtral-8x7B-Instruct-v0.1"
    }

    # Obfuscated URLs and headers
    status_url = base64.b64decode("aHR0cHM6Ly9kdWNrZHVja2dvLmNvbS9kdWNrY2hhdC92MS9zdGF0dXM=").decode("utf-8")
    chat_url = base64.b64decode("aHR0cHM6Ly9kdWNrZHVja2dvLmNvbS9kdWNrY2hhdC92MS9jaGF0").decode("utf-8")
    referer = base64.b64decode("aHR0cHM6Ly9kdWNrZHVja2dvLmNvbS8=").decode("utf-8")
    origin = base64.b64decode("aHR0cHM6Ly9kdWNrZHVja2dvLmNvbQ==").decode("utf-8")
    
    user_agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36'
    headers = {
        'User-Agent': user_agent,
        'Accept': 'text/event-stream',
        'Accept-Language': 'en-US,en;q=0.5',
        'Accept-Encoding': 'gzip, deflate, br, zstd',
        'Referer': referer,
        'Content-Type': 'application/json',
        'Origin': origin,
        'Connection': 'keep-alive',
        'Cookie': 'dcm=3',
        'Sec-Fetch-Dest': 'empty',
        'Sec-Fetch-Mode': 'cors',
        'Sec-Fetch-Site': 'same-origin',
        'Pragma': 'no-cache',
        'TE': 'trailers'
    }

    @classmethod
    async def get_vqd(cls, session: aiohttp.ClientSession) -> Optional[str]:
        try:
            async with session.get(cls.status_url, headers={"x-vqd-accept": "1"}) as response:
                await raise_for_status(response)
                return response.headers.get("x-vqd-4")
        except Exception as e:
            print(f"Error getting VQD: {e}")
            return None

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        connector: aiohttp.BaseConnector = None,
        conversation: Conversation = None,
        return_conversation: bool = False,
        **kwargs
    ) -> AsyncResult:
        async with aiohttp.ClientSession(headers=cls.headers, connector=get_connector(connector, proxy)) as session:
            vqd_4 = None
            if conversation is not None and len(messages) > 1:
                vqd_4 = conversation.vqd_4
                messages = [*conversation.messages, messages[-2], messages[-1]]
            else:
                for _ in range(3):  # Try up to 3 times to get a valid VQD
                    vqd_4 = await cls.get_vqd(session)
                    if vqd_4:
                        break
                    await asyncio.sleep(1)  # Wait a bit before retrying
                
                if not vqd_4:
                    raise Exception("Failed to obtain a valid VQD token")
                
                messages = [messages[-1]]  # Only use the last message for new conversations
            
            payload = {
                'model': cls.get_model(model),
                'messages': [{'role': m['role'], 'content': m['content']} for m in messages]
            }
            
            async with session.post(cls.chat_url, json=payload, headers={"x-vqd-4": vqd_4}) as response:
                await raise_for_status(response)
                if return_conversation:
                    yield Conversation(vqd_4, messages)
                
                async for line in response.content:
                    if line.startswith(b"data: "):
                        chunk = line[6:]
                        if chunk.startswith(b"[DONE]"):
                            break
                        try:
                            data = json.loads(chunk)
                            if "message" in data and data["message"]:
                                yield data["message"]
                        except json.JSONDecodeError:
                            print(f"Failed to decode JSON: {chunk}")

class Conversation(BaseConversation):
    def __init__(self, vqd_4: str, messages: Messages) -> None:
        self.vqd_4 = vqd_4
        self.messages = messages



from __future__ import annotations

from aiohttp import ClientSession, ClientResponseError
from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class LiteIcoding(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://lite.icoding.ink"
    api_endpoint = "/api/v1/gpt/message"
    working = True
    supports_gpt_4 = True
    default_model = "gpt-4o"
    models = [
        'gpt-4o',
        'gpt-4-turbo',
        'claude-3',
        'claude-3.5',
        'gemini-1.5',
    ]

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "Accept": "*/*",
            "Accept-Language": "en-US,en;q=0.9",
            "Authorization": "Bearer null",
            "Connection": "keep-alive",
            "Content-Type": "application/json;charset=utf-8",
            "DNT": "1",
            "Origin": cls.url,
            "Referer": f"{cls.url}/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": (
                "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) "
                "Chrome/126.0.0.0 Safari/537.36"
            ),
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
        }

        data = {
            "model": model,
            "chatId": "-1",
            "messages": [
                {
                    "role": msg["role"],
                    "content": msg["content"],
                    "time": msg.get("time", ""),
                    "attachments": msg.get("attachments", []),
                }
                for msg in messages
            ],
            "plugins": [],
            "systemPrompt": "",
            "temperature": 0.5,
        }

        async with ClientSession(headers=headers) as session:
            try:
                async with session.post(
                    f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy
                ) as response:
                    response.raise_for_status()
                    buffer = ""
                    full_response = ""
                    async for chunk in response.content.iter_any():
                        if chunk:
                            buffer += chunk.decode()
                            while "\n\n" in buffer:
                                part, buffer = buffer.split("\n\n", 1)
                                if part.startswith("data: "):
                                    content = part[6:].strip()
                                    if content and content != "[DONE]":
                                        content = content.strip('"')
                                        full_response += content

                    full_response = full_response.replace('" "', ' ')
                    yield full_response.strip()

            except ClientResponseError as e:
                raise RuntimeError(
                    f"ClientResponseError {e.status}: {e.message}, url={e.request_info.url}, data={data}"
                ) from e

            except Exception as e:
                raise RuntimeError(f"Unexpected error: {str(e)}") from e



from __future__ import annotations

from aiohttp import ClientSession
from ...typing import AsyncResult, Messages
from ..base_provider import AsyncGeneratorProvider

class AiAsk(AsyncGeneratorProvider):
    url = "https://e.aiask.me"
    supports_message_history = True
    supports_gpt_35_turbo = True
    working = False

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "application/json, text/plain, */*",
            "origin": cls.url,
            "referer": f"{cls.url}/chat",
        }
        async with ClientSession(headers=headers) as session:
            data = {
                "continuous": True,
                "id": "fRMSQtuHl91A4De9cCvKD",
                "list": messages,
                "models": "0",
                "prompt": "",
                "temperature": kwargs.get("temperature", 0.5),
                "title": "",
            }
            buffer = ""
            rate_limit = "您的免费额度不够使用这个模型啦，请点击右上角登录继续使用！"
            async with session.post(f"{cls.url}/v1/chat/gpt/", json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for chunk in response.content.iter_any():
                    buffer += chunk.decode()
                    if not rate_limit.startswith(buffer):
                        yield buffer
                        buffer = ""
                    elif buffer == rate_limit:
                        raise RuntimeError("Rate limit reached")


from __future__ import annotations

import json
import base64
import hashlib
import random
from Crypto.Cipher import AES

def pad(data: str) -> bytes:
    # Convert the string to bytes and calculate the number of bytes to pad
    data_bytes = data.encode()
    padding = 16 - (len(data_bytes) % 16)
    # Append the padding bytes with their value
    return data_bytes + bytes([padding] * padding)

def encrypt(data, key):
    salt = ""
    salted = ""
    dx = bytes()

    # Generate salt, as 8 random lowercase letters
    salt = "".join(random.choice("abcdefghijklmnopqrstuvwxyz") for _ in range(8))

    # Our final key and IV come from the key and salt being repeatedly hashed
    for x in range(3):
        dx = hashlib.md5(dx + key.encode() + salt.encode()).digest()
        salted += dx.hex()

    # Pad the data before encryption
    data = pad(data)

    aes = AES.new(
        bytes.fromhex(salted[:64]), AES.MODE_CBC, bytes.fromhex(salted[64:96])
    )

    return json.dumps(
        {
            "ct": base64.b64encode(aes.encrypt(data)).decode(),
            "iv": salted[64:96],
            "s": salt.encode().hex(),
        }
    )

def unpad(data: bytes) -> bytes:
    # Extract the padding value from the last byte and remove padding
    padding_value = data[-1]
    return data[:-padding_value]

def decrypt(data: str, key: str):
    # Parse JSON data
    parsed_data = json.loads(base64.b64decode(data))
    ct = base64.b64decode(parsed_data["ct"])
    iv = bytes.fromhex(parsed_data["iv"])
    salt = bytes.fromhex(parsed_data["s"])

    salted = ''
    dx = b''
    for x in range(3):
        dx = hashlib.md5(dx + key.encode() + salt).digest()
        salted += dx.hex()
        
    aes = AES.new(
        bytes.fromhex(salted[:64]), AES.MODE_CBC, iv
    )

    data = aes.decrypt(ct)
    if data.startswith(b'[{"key":'):
        return unpad(data).decode()


from __future__ import annotations

import json
import random
import requests

from ...typing import Any, CreateResult
from ..base_provider import AbstractProvider


class EasyChat(AbstractProvider):
    url: str              = "https://free.easychat.work"
    supports_stream       = True
    supports_gpt_35_turbo = True
    working               = False

    @staticmethod
    def create_completion(
        model: str,
        messages: list[dict[str, str]],
        stream: bool, **kwargs: Any) -> CreateResult:
        
        active_servers = [
            "https://chat10.fastgpt.me",
            "https://chat9.fastgpt.me",
            "https://chat1.fastgpt.me",
            "https://chat2.fastgpt.me",
            "https://chat3.fastgpt.me",
            "https://chat4.fastgpt.me",
            "https://gxos1h1ddt.fastgpt.me"
        ]

        server  = active_servers[kwargs.get("active_server", random.randint(0, 5))]
        headers = {
            "authority"         : f"{server}".replace("https://", ""),
            "accept"            : "text/event-stream",
            "accept-language"   : "en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3,fa=0.2",
            "content-type"      : "application/json",
            "origin"            : f"{server}",
            "referer"           : f"{server}/",
            "x-requested-with"  : "XMLHttpRequest",
            'plugins'           : '0',
            'sec-ch-ua'         : '"Chromium";v="116", "Not)A;Brand";v="24", "Google Chrome";v="116"',
            'sec-ch-ua-mobile'  : '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest'    : 'empty',
            'sec-fetch-mode'    : 'cors',
            'sec-fetch-site'    : 'same-origin',
            'user-agent'        : 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36',
            'usesearch'         : 'false',
            'x-requested-with'  : 'XMLHttpRequest'
        }

        json_data = {
            "messages"          : messages,
            "stream"            : stream,
            "model"             : model,
            "temperature"       : kwargs.get("temperature", 0.5),
            "presence_penalty"  : kwargs.get("presence_penalty", 0),
            "frequency_penalty" : kwargs.get("frequency_penalty", 0),
            "top_p"             : kwargs.get("top_p", 1)
        }

        session = requests.Session()
        # init cookies from server
        session.get(f"{server}/")

        response = session.post(f"{server}/api/openai/v1/chat/completions",
            headers=headers, json=json_data, stream=stream)

        if response.status_code != 200:
            raise Exception(f"Error {response.status_code} from server : {response.reason}")
        if not stream:
            json_data = response.json()

            if "choices" in json_data:
                yield json_data["choices"][0]["message"]["content"]
            else:
                raise Exception("No response from server")

        else:
                
            for chunk in response.iter_lines():
                    
                if b"content" in chunk:
                    splitData = chunk.decode().split("data:")

                    if len(splitData) > 1:
                        yield json.loads(splitData[1])["choices"][0]["delta"]["content"]


from __future__ import annotations

import requests

from .needs_auth.Openai import Openai
from ..typing import AsyncResult, Messages

class Ollama(Openai):
    label = "Ollama"
    url = "https://ollama.com"
    needs_auth = False
    working = True

    @classmethod
    def get_models(cls):
        if not cls.models:
            url = 'http://127.0.0.1:11434/api/tags'
            models = requests.get(url).json()["models"]
            cls.models = [model['name'] for model in models]
            cls.default_model = cls.models[0]
        return cls.models

    @classmethod
    def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        api_base: str = "http://localhost:11434/v1",
        **kwargs
    ) -> AsyncResult:
        return super().create_async_generator(
            model, messages, api_base=api_base, **kwargs
        )


from __future__ import annotations

import json
import asyncio
from aiohttp import ClientSession, ClientTimeout, ClientError
from typing import AsyncGenerator

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin


class FreeNetfly(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://free.netfly.top"
    api_endpoint = "/api/openai/v1/chat/completions"
    working = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = True
    default_model = 'gpt-3.5-turbo'
    models = [
        'gpt-3.5-turbo',
        'gpt-4',
    ]

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "application/json, text/event-stream",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "dnt": "1",
            "origin": cls.url,
            "referer": f"{cls.url}/",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
        }
        data = {
            "messages": messages,
            "stream": True,
            "model": model,
            "temperature": 0.5,
            "presence_penalty": 0,
            "frequency_penalty": 0,
            "top_p": 1
        }
        
        max_retries = 3
        retry_delay = 1

        for attempt in range(max_retries):
            try:
                async with ClientSession(headers=headers) as session:
                    timeout = ClientTimeout(total=60)
                    async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy, timeout=timeout) as response:
                        response.raise_for_status()
                        async for chunk in cls._process_response(response):
                            yield chunk
                        return  # If successful, exit the function
            except (ClientError, asyncio.TimeoutError) as e:
                if attempt == max_retries - 1:
                    raise  # If all retries failed, raise the last exception
                await asyncio.sleep(retry_delay)
                retry_delay *= 2  # Exponential backoff

    @classmethod
    async def _process_response(cls, response) -> AsyncGenerator[str, None]:
        buffer = ""
        async for line in response.content:
            buffer += line.decode('utf-8')
            if buffer.endswith('\n\n'):
                for subline in buffer.strip().split('\n'):
                    if subline.startswith('data: '):
                        if subline == 'data: [DONE]':
                            return
                        try:
                            data = json.loads(subline[6:])
                            content = data['choices'][0]['delta'].get('content')
                            if content:
                                yield content
                        except json.JSONDecodeError:
                            print(f"Failed to parse JSON: {subline}")
                        except KeyError:
                            print(f"Unexpected JSON structure: {data}")
                buffer = ""
        
        # Process any remaining data in the buffer
        if buffer:
            for subline in buffer.strip().split('\n'):
                if subline.startswith('data: ') and subline != 'data: [DONE]':
                    try:
                        data = json.loads(subline[6:])
                        content = data['choices'][0]['delta'].get('content')
                        if content:
                            yield content
                    except (json.JSONDecodeError, KeyError):
                        pass




from __future__ import annotations

from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider
from ..requests import get_args_from_browser
from ..webdriver import WebDriver

class Aura(AsyncGeneratorProvider):
    url = "https://openchat.team"
    working = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        temperature: float = 0.5,
        max_tokens: int = 8192,
        webdriver: WebDriver = None,
        **kwargs
    ) -> AsyncResult:
        args = get_args_from_browser(cls.url, webdriver, proxy)
        async with ClientSession(**args) as session:
            new_messages = []
            system_message = []
            for message in messages:
                if message["role"] == "system":
                    system_message.append(message["content"])
                else:
                    new_messages.append(message)
            data = {
                "model": {
                    "id": "openchat_v3.2_mistral",
                    "name": "OpenChat Aura",
                    "maxLength": 24576,
                    "tokenLimit": max_tokens
                },
                "messages": new_messages,
                "key": "",
                "prompt": "\n".join(system_message),
                "temperature": temperature
            }
            async with session.post(f"{cls.url}/api/chat", json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for chunk in response.content.iter_any():
                    yield chunk.decode(error="ignore")


from .Gemini        import Gemini
from .Raycast       import Raycast
from .Theb          import Theb
from .ThebApi       import ThebApi
from .OpenaiChat    import OpenaiChat
from .Poe           import Poe
from .Openai        import Openai
from .Groq          import Groq
from .OpenRouter    import OpenRouter
from .OpenaiAccount import OpenaiAccount
from .PerplexityApi import PerplexityApi


from __future__ import annotations

from ..locals.models import get_models
try:
    from ..locals.provider import LocalProvider
    has_requirements = True
except ImportError:
    has_requirements = False

from ..typing import Messages, CreateResult
from ..providers.base_provider import AbstractProvider, ProviderModelMixin
from ..errors import MissingRequirementsError

class Local(AbstractProvider, ProviderModelMixin):
    label = "GPT4All"
    working = True
    supports_message_history = True
    supports_system_message = True
    supports_stream = True

    @classmethod
    def get_models(cls):
        if not cls.models:
            cls.models = list(get_models())
            cls.default_model = cls.models[0]
        return cls.models

    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: Messages,
        stream: bool,
        **kwargs
    ) -> CreateResult:
        if not has_requirements:
            raise MissingRequirementsError('Install "gpt4all" package | pip install -U g4f[local]')
        return LocalProvider.create_completion(
            cls.get_model(model),
            messages,
            stream,
            **kwargs
        )


from __future__  import annotations

from dataclasses import dataclass

from .Provider import IterListProvider, ProviderType
from .Provider import (
    AI365VIP,
    Allyfy,
    Bing,
    Blackbox,
    ChatGot,
    Chatgpt4o,
    Chatgpt4Online,
    ChatgptFree,
    DDG,
    DeepInfra,
    DeepInfraImage,
    FreeChatgpt,
    FreeGpt,
    FreeNetfly,
    Gemini,
    GeminiPro,
    GeminiProChat,
    GigaChat,
    HuggingChat,
    HuggingFace,
    Koala,
    Liaobots,
    LiteIcoding,
    MagickPenAsk,
    MagickPenChat,
    Marsyoo,
    MetaAI,
    OpenaiChat,
    PerplexityLabs,
    Phind,
    Pi,
    Pizzagpt,
    Reka,
    Replicate,
    ReplicateHome,
    Vercel,
    TeachAnything,
    You,
)


@dataclass(unsafe_hash=True)
class Model:
    """
    Represents a machine learning model configuration.

    Attributes:
        name (str): Name of the model.
        base_provider (str): Default provider for the model.
        best_provider (ProviderType): The preferred provider for the model, typically with retry logic.
    """
    name: str
    base_provider: str
    best_provider: ProviderType = None

    @staticmethod
    def __all__() -> list[str]:
        """Returns a list of all model names."""
        return _all_models

default = Model(
    name          = "",
    base_provider = "",
    best_provider = IterListProvider([
        FreeChatgpt,
        AI365VIP,
        Chatgpt4o,
        DDG,
        ChatgptFree,
        Koala,
        Pizzagpt,
        Bing,
        OpenaiChat,
    ])
)

# GPT-3.5 too, but all providers supports long requests and responses
gpt_35_long = Model(
    name          = 'gpt-3.5-turbo',
    base_provider = 'openai',
    best_provider = IterListProvider([
        Koala,
        ChatgptFree,
        FreeChatgpt,
        DDG,
        AI365VIP,
        Pizzagpt,
        FreeGpt,
        You,
        Allyfy,
    ])
)

############
### Text ###
############

### OpenAI ###
### GPT-3.5 / GPT-4 ###
# gpt-3.5
gpt_35_turbo = Model(
    name          = 'gpt-3.5-turbo',
    base_provider = 'openai',
    best_provider = IterListProvider([
        Koala,
        ChatgptFree,
        FreeChatgpt,
        AI365VIP,
        Pizzagpt,
        FreeGpt,
        You,
        OpenaiChat,
        Allyfy,
    ])
)

# gpt-4
gpt_4 = Model(
    name          = 'gpt-4',
    base_provider = 'openai',
    best_provider = IterListProvider([
        Bing, Chatgpt4Online
    ])
)

gpt_4_turbo = Model(
    name          = 'gpt-4-turbo',
    base_provider = 'openai',
    best_provider = IterListProvider([
        Bing, Liaobots, LiteIcoding
    ])
)
gpt_4o = Model(
    name          = 'gpt-4o',
    base_provider = 'openai',
    best_provider = IterListProvider([
        Chatgpt4o, AI365VIP, Marsyoo, Liaobots, LiteIcoding, OpenaiChat, MagickPenAsk,
    ])
)

gpt_4o_mini = Model(
    name          = 'gpt-4o-mini',
    base_provider = 'openai',
    best_provider = IterListProvider([
        DDG, FreeNetfly, MagickPenChat, Liaobots, OpenaiChat,
    ])
)


### GigaChat ###
gigachat = Model(
    name          = 'GigaChat:latest',
    base_provider = 'gigachat',
    best_provider = GigaChat
)


### Meta ###
meta = Model(
    name          = "meta",
    base_provider = "meta",
    best_provider = MetaAI
)

llama_3_8b_instruct = Model(
    name          = "meta-llama/Meta-Llama-3-8B-Instruct",
    base_provider = "meta",
    best_provider = IterListProvider([DeepInfra, PerplexityLabs, Replicate])
)

llama_3_70b_instruct = Model(
    name          = "meta-llama/Meta-Llama-3-70B-Instruct",
    base_provider = "meta",
    best_provider = IterListProvider([DeepInfra, Replicate, DDG, PerplexityLabs, HuggingChat])
)

llama_3_70b_chat_hf = Model(
    name          = "meta-llama/Llama-3-70b-chat-hf",
    base_provider = "meta",
    best_provider = IterListProvider([DDG])
)

llama_3_1_70b_instruct = Model(
    name          = "meta-llama/Meta-Llama-3.1-70B-Instruct",
    base_provider = "meta",
    best_provider = IterListProvider([DeepInfra, HuggingChat, HuggingFace])
)

llama_3_1_405b_instruct_FP8 = Model(
    name          = "meta-llama/Meta-Llama-3.1-405B-Instruct-FP8",
    base_provider = "meta",
    best_provider = IterListProvider([DeepInfra, HuggingChat, HuggingFace])
)


### Mistral ###
mixtral_8x7b = Model(
    name          = "mistralai/Mixtral-8x7B-Instruct-v0.1",
    base_provider = "huggingface",
    best_provider = IterListProvider([DeepInfra, DDG, HuggingFace, PerplexityLabs, HuggingChat, ReplicateHome])
)

mistral_7b_v02 = Model(
    name          = "mistralai/Mistral-7B-Instruct-v0.2",
    base_provider = "huggingface",
    best_provider = IterListProvider([DeepInfra, HuggingFace, HuggingChat])
)


### NousResearch ###
Nous_Hermes_2_Mixtral_8x7B_DPO = Model(
    name          = "NousResearch/Nous-Hermes-2-Mixtral-8x7B-DPO",
    base_provider = "NousResearch",
    best_provider = IterListProvider([HuggingFace, HuggingChat])
)


### 01-ai ###
Yi_1_5_34B_chat = Model(
    name          = "01-ai/Yi-1.5-34B-Chat",
    base_provider = "01-ai",
    best_provider = IterListProvider([HuggingFace, HuggingChat])
)


### Microsoft ###
Phi_3_mini_4k_instruct = Model(
    name          = "microsoft/Phi-3-mini-4k-instruct",
    base_provider = "Microsoft",
    best_provider = IterListProvider([HuggingFace, HuggingChat])
)


### Google ###
# gemini
gemini = Model(
    name          = 'gemini',
    base_provider = 'Google',
    best_provider = Gemini
)

gemini_pro = Model(
    name          = 'gemini-pro',
    base_provider = 'Google',
    best_provider = IterListProvider([GeminiProChat, GeminiPro, ChatGot, Liaobots, LiteIcoding, You])
)

gemini_flash = Model(
    name          = 'gemini-flash',
    base_provider = 'Google',
    best_provider = IterListProvider([Liaobots])
)

gemini_1_5 = Model(
    name          = 'gemini-1.5',
    base_provider = 'Google',
    best_provider = IterListProvider([LiteIcoding])
)

# gemma
gemma_2b_it = Model(
    name          = 'gemma-2b-it',
    base_provider = 'Google',
    best_provider = IterListProvider([ReplicateHome])
)

gemma_2_9b_it = Model(
    name          = 'gemma-2-9b-it',
    base_provider = 'Google',
    best_provider = IterListProvider([PerplexityLabs])
)

gemma_2_27b_it = Model(
    name          = 'gemma-2-27b-it',
    base_provider = 'Google',
    best_provider = IterListProvider([PerplexityLabs])
)


### Anthropic ###
claude_2 = Model(
    name          = 'claude-2',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([You])
)

claude_2_0 = Model(
    name          = 'claude-2.0',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_2_1 = Model(
    name          = 'claude-2.1',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_opus = Model(
    name          = 'claude-3-opus',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_sonnet = Model(
    name          = 'claude-3-sonnet',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_5_sonnet = Model(
    name          = 'claude-3-5-sonnet',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([Liaobots])
)

claude_3_haiku = Model(
    name          = 'claude-3-haiku',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([DDG, AI365VIP, Liaobots])
)

claude_3 = Model(
    name          = 'claude-3',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([LiteIcoding])
)

claude_3_5 = Model(
    name          = 'claude-3.5',
    base_provider = 'Anthropic',
    best_provider = IterListProvider([LiteIcoding])
)


### Reka AI ###
reka_core = Model(
    name = 'reka-core',
    base_provider = 'Reka AI',
    best_provider = Reka
)


### NVIDIA ###
nemotron_4_340b_instruct = Model(
    name = 'nemotron-4-340b-instruct',
    base_provider = 'NVIDIA',
    best_provider = IterListProvider([PerplexityLabs])
)


### Blackbox ###
blackbox = Model(
    name = 'blackbox',
    base_provider = 'Blackbox',
    best_provider = Blackbox
)


### Databricks ###
dbrx_instruct = Model(
    name = 'databricks/dbrx-instruct',
    base_provider = 'Databricks',
    best_provider = IterListProvider([DeepInfra])
)


### CohereForAI ###
command_r_plus = Model(
    name = 'CohereForAI/c4ai-command-r-plus',
    base_provider = 'CohereForAI',
    best_provider = IterListProvider([HuggingChat])
)


### iFlytek ###
SparkDesk_v1_1 = Model(
    name = 'SparkDesk-v1.1',
    base_provider = 'iFlytek',
    best_provider = IterListProvider([FreeChatgpt])
)


### DeepSeek ###
deepseek_coder = Model(
    name = 'deepseek-coder',
    base_provider = 'DeepSeek',
    best_provider = IterListProvider([FreeChatgpt])
)

deepseek_chat = Model(
    name = 'deepseek-chat',
    base_provider = 'DeepSeek',
    best_provider = IterListProvider([FreeChatgpt])
)


### Qwen ###
Qwen2_7B_instruct = Model(
    name = 'Qwen2-7B-Instruct',
    base_provider = 'Qwen',
    best_provider = IterListProvider([FreeChatgpt])
)


### Zhipu AI ###
glm4_9B_chat = Model(
    name = 'glm4-9B-chat',
    base_provider = 'Zhipu AI',
    best_provider = IterListProvider([FreeChatgpt])
)

chatglm3_6B = Model(
    name = 'chatglm3-6B',
    base_provider = 'Zhipu AI',
    best_provider = IterListProvider([FreeChatgpt])
)


### 01-ai ###
Yi_1_5_9B_chat = Model(
    name = 'Yi-1.5-9B-Chat',
    base_provider = '01-ai',
    best_provider = IterListProvider([FreeChatgpt])
)


### Other ###
pi = Model(
    name = 'pi',
    base_provider = 'inflection',
    best_provider = Pi
)


#############
### Image ###
#############

### Stability AI ###
sdxl = Model(
    name = 'stability-ai/sdxl',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([DeepInfraImage])
    
)

stable_diffusion_3 = Model(
    name = 'stability-ai/stable-diffusion-3',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([ReplicateHome])
    
)

sdxl_lightning_4step = Model(
    name = 'bytedance/sdxl-lightning-4step',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([ReplicateHome])
    
)

playground_v2_5_1024px_aesthetic = Model(
    name = 'playgroundai/playground-v2.5-1024px-aesthetic',
    base_provider = 'Stability AI',
    best_provider = IterListProvider([ReplicateHome])
    
)

class ModelUtils:
    """
    Utility class for mapping string identifiers to Model instances.

    Attributes:
        convert (dict[str, Model]): Dictionary mapping model string identifiers to Model instances.
    """
    convert: dict[str, Model] = {
    
        ############
        ### Text ###
        ############
        
        ### OpenAI ###
        ### GPT-3.5 / GPT-4 ###
        # gpt-3.5
        'gpt-3.5-turbo': gpt_35_turbo,
        'gpt-3.5-long': gpt_35_long,

        # gpt-4
        'gpt-4o'         : gpt_4o,
        'gpt-4o-mini'    : gpt_4o_mini,
        'gpt-4'          : gpt_4,
        'gpt-4-turbo'    : gpt_4_turbo,
        
        ### Meta ###
        "meta-ai": meta,
        
        'llama-3-8b-instruct': llama_3_8b_instruct,
        'llama-3-70b-instruct': llama_3_70b_instruct,
        'llama-3-70b-chat': llama_3_70b_chat_hf, 
        'llama-3-70b-instruct': llama_3_70b_instruct, 
        
        'llama-3.1-70b': llama_3_1_70b_instruct,
        'llama-3.1-405b': llama_3_1_405b_instruct_FP8,
        'llama-3.1-70b-instruct': llama_3_1_70b_instruct,
        'llama-3.1-405b-instruct': llama_3_1_405b_instruct_FP8,
        
        ### Mistral (Opensource) ###
        'mixtral-8x7b': mixtral_8x7b,
        'mistral-7b-v02': mistral_7b_v02,
        
        ### NousResearch ###
        'Nous-Hermes-2-Mixtral-8x7B-DPO': Nous_Hermes_2_Mixtral_8x7B_DPO,

        ### 01-ai ###
        'Yi-1.5-34b-chat': Yi_1_5_34B_chat,
        
        ### Microsoft ###
        'Phi-3-mini-4k-instruct': Phi_3_mini_4k_instruct,

        ### Google ###
        # gemini
        'gemini': gemini,
        'gemini-pro': gemini_pro,
        'gemini-pro': gemini_1_5,
        'gemini-flash': gemini_flash,
        
        # gemma
        'gemma-2b': gemma_2b_it,
        'gemma-2-9b': gemma_2_9b_it,
        'gemma-2-27b': gemma_2_27b_it,

        ### Anthropic ###
        'claude-2': claude_2,
        'claude-2.0': claude_2_0,
        'claude-2.1': claude_2_1,
        
        'claude-3-opus': claude_3_opus,
        'claude-3-sonnet': claude_3_sonnet,
        'claude-3-5-sonnet': claude_3_5_sonnet,
        'claude-3-haiku': claude_3_haiku,
        
        'claude-3-opus': claude_3,
        'claude-3-5-sonnet': claude_3_5,
        
        

        ### Reka AI ###
        'reka': reka_core,

        ### NVIDIA ###
        'nemotron-4-340b-instruct': nemotron_4_340b_instruct,
        
        ### Blackbox ###
        'blackbox': blackbox,
        
        ### CohereForAI ###
        'command-r+': command_r_plus,
        
        ### Databricks ###
        'dbrx-instruct': dbrx_instruct,

        ### GigaChat ###
        'gigachat': gigachat,
        
        ### iFlytek ###
        'SparkDesk-v1.1': SparkDesk_v1_1,
        
        ### DeepSeek ###
        'deepseek-coder': deepseek_coder,
        'deepseek-chat': deepseek_chat,
        
        ### Qwen ###
        'Qwen2-7b-instruct': Qwen2_7B_instruct,
        
        ### Zhipu AI ###
        'glm4-9b-chat': glm4_9B_chat,
        'chatglm3-6b': chatglm3_6B,
        
        ### 01-ai ###
        'Yi-1.5-9b-chat': Yi_1_5_9B_chat,
        
        # Other
        'pi': pi,
        
        #############
        ### Image ###
        #############
        
        ### Stability AI ###
        'sdxl': sdxl,
        'stable-diffusion-3': stable_diffusion_3,
        
        ### ByteDance ###
        'sdxl-lightning': sdxl_lightning_4step,
        
        ### Playground ###
        'playground-v2.5': playground_v2_5_1024px_aesthetic,

    }

_all_models = list(ModelUtils.convert.keys())



from __future__ import annotations

import time
import random

from ...typing import CreateResult, Messages
from ..base_provider import AbstractProvider
from ..helper import format_prompt, get_random_string
from ...webdriver import WebDriver, WebDriverSession, element_send_text
from ... import debug

class AItianhuSpace(AbstractProvider):
    url = "https://chat3.aiyunos.top/"
    working = True
    supports_stream = True
    supports_gpt_35_turbo = True
    _domains = ["aitianhu.com", "aitianhu1.top"]

    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: Messages,
        stream: bool,
        domain: str = None,
        proxy: str = None,
        timeout: int = 120,
        webdriver: WebDriver = None,
        headless: bool = True,
        **kwargs
    ) -> CreateResult:
        if not model:
            model = "gpt-3.5-turbo"
        if not domain:
            rand = get_random_string(6)
            domain = random.choice(cls._domains)
            domain = f"{rand}.{domain}"
        if debug.logging:
            print(f"AItianhuSpace | using domain: {domain}")
        url = f"https://{domain}"
        prompt = format_prompt(messages)

        with WebDriverSession(webdriver, "", headless=headless, proxy=proxy) as driver:
            from selenium.webdriver.common.by import By
            from selenium.webdriver.support.ui import WebDriverWait
            from selenium.webdriver.support import expected_conditions as EC

            wait = WebDriverWait(driver, timeout)

            # Bypass devtools detection
            driver.get("https://blank.page/")
            wait.until(EC.visibility_of_element_located((By.ID, "sheet")))
            driver.execute_script(f"""
    document.getElementById('sheet').addEventListener('click', () => {{
        window.open(arguments[0]);
    }});
            """, url)
            driver.find_element(By.ID, "sheet").click()
            time.sleep(10)

            original_window = driver.current_window_handle
            for window_handle in driver.window_handles:
                if window_handle != original_window:
                    driver.close()
                    driver.switch_to.window(window_handle)
                    break

            # Wait for page load
            wait.until(EC.visibility_of_element_located((By.CSS_SELECTOR, "textarea.n-input__textarea-el")))

            # Register hook in XMLHttpRequest
            script = """
const _http_request_open = XMLHttpRequest.prototype.open;
window._last_message = window._message = "";
window._loadend = false;
XMLHttpRequest.prototype.open = function(method, url) {
    if (url == "/api/chat-process") {
        this.addEventListener("progress", (event) => {
            const lines = this.responseText.split("\\n");
            try {
                window._message = JSON.parse(lines[lines.length-1])["text"];
            } catch(e) { }
        });
        this.addEventListener("loadend", (event) => {
            window._loadend = true;
        });
    }
    return _http_request_open.call(this, method, url);
}
"""
            driver.execute_script(script)

            # Submit prompt
            element_send_text(driver.find_element(By.CSS_SELECTOR, "textarea.n-input__textarea-el"), prompt)

            # Read response
            while True:
                chunk = driver.execute_script("""
if (window._message && window._message != window._last_message) {
    try {
        return window._message.substring(window._last_message.length);
    } finally {
        window._last_message = window._message;
    }
}
if (window._loadend) {
    return null;
}
return "";
""")
                if chunk:
                    yield chunk
                elif chunk != "":
                    break
                else:
                    time.sleep(0.1)


import random
import hashlib
import json
import base64
from datetime import datetime, timezone


def generate_proof_token(required: bool, seed: str = "", difficulty: str = "", user_agent: str = None, proofTokens: list = None):
    if not required:
        return

    if proofTokens:
        config = proofTokens[-1]
    else:
        screen = random.choice([3008, 4010, 6000]) * random.choice([1, 2, 4])
        # Get current UTC time
        now_utc = datetime.now(timezone.utc)
        parse_time = now_utc.strftime('%a, %d %b %Y %H:%M:%S GMT')
        config = [
            screen, parse_time,
            None, 0, user_agent,
            "https://tcr9i.chat.openai.com/v2/35536E1E-65B4-4D96-9D97-6ADB7EFF8147/api.js",
            "dpl=1440a687921de39ff5ee56b92807faaadce73f13","en","en-US",
            None,
            "plugins−[object PluginArray]",
            random.choice(["_reactListeningcfilawjnerp", "_reactListening9ne2dfo1i47", "_reactListening410nzwhan2a"]),
            random.choice(["alert", "ontransitionend", "onprogress"])
        ]

    diff_len = len(difficulty)
    for i in range(100000):
        config[3] = i
        json_data = json.dumps(config)
        base = base64.b64encode(json_data.encode()).decode()
        hash_value = hashlib.sha3_512((seed + base).encode()).digest()

        if hash_value.hex()[:diff_len] <= difficulty:
            return "gAAAAAB" + base

    fallback_base = base64.b64encode(f'"{seed}"'.encode()).decode()
    return "gAAAAABwQ8Lk5FbGpA2NcR9dShT6gYjU7VxZ4D" + fallback_base



from __future__ import annotations

import json, requests, re

from curl_cffi      import requests as cf_reqs
from ..typing       import CreateResult, Messages
from .base_provider import ProviderModelMixin, AbstractProvider
from .helper        import format_prompt

class HuggingChat(AbstractProvider, ProviderModelMixin):
    url             = "https://huggingface.co/chat"
    working         = True
    supports_stream = True
    default_model   = "mistralai/Mixtral-8x7B-Instruct-v0.1"
    models = [
        'meta-llama/Meta-Llama-3.1-70B-Instruct',
        'meta-llama/Meta-Llama-3.1-405B-Instruct-FP8',
        'CohereForAI/c4ai-command-r-plus',
        'mistralai/Mixtral-8x7B-Instruct-v0.1',
        'NousResearch/Nous-Hermes-2-Mixtral-8x7B-DPO',
        '01-ai/Yi-1.5-34B-Chat',
        'mistralai/Mistral-7B-Instruct-v0.2',
        'microsoft/Phi-3-mini-4k-instruct',
    ]
    
    model_aliases = {
        "mistralai/Mistral-7B-Instruct-v0.1": "mistralai/Mistral-7B-Instruct-v0.2"
    }

    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: Messages,
        stream: bool,
        **kwargs) -> CreateResult:
        
        if (model in cls.models) :
            
            session = cf_reqs.Session()
            session.headers = {
                'accept': '*/*',
                'accept-language': 'en',
                'cache-control': 'no-cache',
                'origin': 'https://huggingface.co',
                'pragma': 'no-cache',
                'priority': 'u=1, i',
                'referer': 'https://huggingface.co/chat/',
                'sec-ch-ua': '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
                'sec-ch-ua-mobile': '?0',
                'sec-ch-ua-platform': '"macOS"',
                'sec-fetch-dest': 'empty',
                'sec-fetch-mode': 'cors',
                'sec-fetch-site': 'same-origin',
                'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36',
            }

            print(model)
            json_data = {
                'model': model,
            }

            response = session.post('https://huggingface.co/chat/conversation', json=json_data)
            conversationId = response.json()['conversationId']

            response = session.get(f'https://huggingface.co/chat/conversation/{conversationId}/__data.json?x-sveltekit-invalidated=01',)

            data: list = (response.json())["nodes"][1]["data"]
            keys: list[int] = data[data[0]["messages"]]
            message_keys: dict = data[keys[0]]
            messageId: str = data[message_keys["id"]]

            settings = {
                "inputs":format_prompt(messages),
                "id":messageId,
                "is_retry":False,
                "is_continue":False,
                "web_search":False,
                "tools":[]
            }

            headers = {
                'accept': '*/*',
                'accept-language': 'en',
                'cache-control': 'no-cache',
                'origin': 'https://huggingface.co',
                'pragma': 'no-cache',
                'priority': 'u=1, i',
                'referer': f'https://huggingface.co/chat/conversation/{conversationId}',
                'sec-ch-ua': '"Not)A;Brand";v="99", "Google Chrome";v="127", "Chromium";v="127"',
                'sec-ch-ua-mobile': '?0',
                'sec-ch-ua-platform': '"macOS"',
                'sec-fetch-dest': 'empty',
                'sec-fetch-mode': 'cors',
                'sec-fetch-site': 'same-origin',
                'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36',
            }


            files = {
                'data': (None,  json.dumps(settings, separators=(',', ':'))),
            }

            response = requests.post(f'https://huggingface.co/chat/conversation/{conversationId}',
                cookies=session.cookies,
                headers=headers,
                files=files,
            )

            first_token = True
            for line in response.iter_lines():
                line = json.loads(line)
                
                if "type" not in line:
                    raise RuntimeError(f"Response: {line}")
                
                elif line["type"] == "stream":
                    token = line["token"]
                    if first_token:
                        token = token.lstrip().replace('\u0000', '')
                        first_token = False
                    
                    else:
                        token = token.replace('\u0000', '')

                    yield (token)
                
                elif line["type"] == "finalAnswer":
                    break


from __future__ import annotations

import time
import random
import string

from ..typing import Union, Iterator, Messages, ImageType
from ..providers.types import BaseProvider, ProviderType, FinishReason
from ..providers.conversation import BaseConversation
from ..image import ImageResponse as ImageProviderResponse
from ..errors import NoImageResponseError
from .stubs import ChatCompletion, ChatCompletionChunk, Image, ImagesResponse
from .image_models import ImageModels
from .types import IterResponse, ImageProvider
from .types import Client as BaseClient
from .service import get_model_and_provider, get_last_provider
from .helper import find_stop, filter_json, filter_none

def iter_response(
    response: iter[str],
    stream: bool,
    response_format: dict = None,
    max_tokens: int = None,
    stop: list = None
) -> IterResponse:
    content = ""
    finish_reason = None
    completion_id = ''.join(random.choices(string.ascii_letters + string.digits, k=28))
    for idx, chunk in enumerate(response):
        if isinstance(chunk, FinishReason):
            finish_reason = chunk.reason
            break
        elif isinstance(chunk, BaseConversation):
            yield chunk
            continue
        content += str(chunk)
        if max_tokens is not None and idx + 1 >= max_tokens:
            finish_reason = "length"
        first, content, chunk = find_stop(stop, content, chunk if stream else None)
        if first != -1:
            finish_reason = "stop"
        if stream:
            yield ChatCompletionChunk(chunk, None, completion_id, int(time.time()))
        if finish_reason is not None:
            break
    finish_reason = "stop" if finish_reason is None else finish_reason
    if stream:
        yield ChatCompletionChunk(None, finish_reason, completion_id, int(time.time()))
    else:
        if response_format is not None and "type" in response_format:
            if response_format["type"] == "json_object":
                content = filter_json(content)
        yield ChatCompletion(content, finish_reason, completion_id, int(time.time()))

def iter_append_model_and_provider(response: IterResponse) -> IterResponse:
    last_provider = None
    for chunk in response:
        last_provider = get_last_provider(True) if last_provider is None else last_provider
        chunk.model = last_provider.get("model")
        chunk.provider =  last_provider.get("name")
        yield chunk

class Client(BaseClient):
    def __init__(
        self,
        provider: ProviderType = None,
        image_provider: ImageProvider = None,
        **kwargs
    ) -> None:
        super().__init__(**kwargs)
        self.chat: Chat = Chat(self, provider)
        self.images: Images = Images(self, image_provider)

class Completions():
    def __init__(self, client: Client, provider: ProviderType = None):
        self.client: Client = client
        self.provider: ProviderType = provider

    def create(
        self,
        messages: Messages,
        model: str,
        provider: ProviderType = None,
        stream: bool = False,
        proxy: str = None,
        response_format: dict = None,
        max_tokens: int = None,
        stop: Union[list[str], str] = None,
        api_key: str = None,
        ignored  : list[str] = None,
        ignore_working: bool = False,
        ignore_stream: bool = False,
        **kwargs
    ) -> Union[ChatCompletion, Iterator[ChatCompletionChunk]]:
        model, provider = get_model_and_provider(
            model,
            self.provider if provider is None else provider,
            stream,
            ignored,
            ignore_working,
            ignore_stream,
        )
        
        stop = [stop] if isinstance(stop, str) else stop
        response = provider.create_completion(
            model, messages,
            stream=stream,            
            **filter_none(
                proxy=self.client.get_proxy() if proxy is None else proxy,
                max_tokens=max_tokens,
                stop=stop,
                api_key=self.client.api_key if api_key is None else api_key
            ),
            **kwargs
        )
        response = iter_response(response, stream, response_format, max_tokens, stop)
        response = iter_append_model_and_provider(response)
        return response if stream else next(response)

class Chat():
    completions: Completions

    def __init__(self, client: Client, provider: ProviderType = None):
        self.completions = Completions(client, provider)

def iter_image_response(response: Iterator) -> Union[ImagesResponse, None]:
    for chunk in list(response):
        if isinstance(chunk, ImageProviderResponse):
            return ImagesResponse([Image(image) for image in chunk.get_list()])

def create_image(client: Client, provider: ProviderType, prompt: str, model: str = "", **kwargs) -> Iterator:


    if isinstance(provider, type) and provider.__name__ == "You":
        kwargs["chat_mode"] = "create"
    else:
        prompt = f"create a image with: {prompt}"
    return provider.create_completion(
        model,
        [{"role": "user", "content": prompt}],
        stream=True,
        proxy=client.get_proxy(),
        **kwargs
    )

class Images():
    def __init__(self, client: Client, provider: ImageProvider = None):
        self.client: Client = client
        self.provider: ImageProvider = provider
        self.models: ImageModels = ImageModels(client)

    def generate(self, prompt, model: str = None, **kwargs) -> ImagesResponse:
        provider = self.models.get(model, self.provider)
        if isinstance(provider, type) and issubclass(provider, BaseProvider):
            response = create_image(self.client, provider, prompt, **kwargs)
        else:
            response = list(provider.create(prompt))
        image = iter_image_response(response)
        if image is None:
            raise NoImageResponseError()
        return image

    def create_variation(self, image: ImageType, model: str = None, **kwargs):
        provider = self.models.get(model, self.provider)
        result = None
        if isinstance(provider, type) and issubclass(provider, BaseProvider):
            response = provider.create_completion(
                "",
                [{"role": "user", "content": "create a image like this"}],
                True,
                image=image,
                proxy=self.client.get_proxy(),
                **kwargs
            )
            result = iter_image_response(response)
        if result is None:
            raise NoImageResponseError()
        return result


from __future__ import annotations

from aiohttp import ClientSession
import json

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider
from .helper import format_prompt


class Allyfy(AsyncGeneratorProvider):
    url = "https://chatbot.allyfy.chat"
    api_endpoint = "/api/v1/message/stream/super/chat"
    working = True
    supports_gpt_35_turbo = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "text/event-stream",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json;charset=utf-8",
            "dnt": "1",
            "origin": "https://www.allyfy.chat",
            "priority": "u=1, i",
            "referer": "https://www.allyfy.chat/",
            "referrer": "https://www.allyfy.chat",
            'sec-ch-ua': '"Not/A)Brand";v="8", "Chromium";v="126"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-site",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
        }
        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "messages": [{"content": prompt, "role": "user"}],
                "content": prompt,
                "baseInfo": {
                    "clientId": "q08kdrde1115003lyedfoir6af0yy531",
                    "pid": "38281",
                    "channelId": "100000",
                    "locale": "en-US",
                    "localZone": 180,
                    "packageName": "com.cch.allyfy.webh",
                }
            }
            async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy) as response:
                response.raise_for_status()
                full_response = []
                async for line in response.content:
                    line = line.decode().strip()
                    if line.startswith("data:"):
                        data_content = line[5:]
                        if data_content == "[DONE]":
                            break
                        try:
                            json_data = json.loads(data_content)
                            if "content" in json_data:
                                full_response.append(json_data["content"])
                        except json.JSONDecodeError:
                            continue
                yield "".join(full_response)



from __future__ import annotations

from .Openai import Openai
from ...typing import AsyncResult, Messages

class Groq(Openai):
    label = "Groq"
    url = "https://console.groq.com/playground"
    working = True
    default_model = "mixtral-8x7b-32768"
    models = ["mixtral-8x7b-32768", "llama2-70b-4096", "gemma-7b-it"]
    model_aliases = {"mixtral-8x7b": "mixtral-8x7b-32768", "llama2-70b": "llama2-70b-4096"}

    @classmethod
    def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        api_base: str = "https://api.groq.com/openai/v1",
        **kwargs
    ) -> AsyncResult:
        return super().create_async_generator(
            model, messages, api_base=api_base, **kwargs
        )


from __future__ import annotations

import json
import time
import hashlib
from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import get_random_hex, get_random_string
from ..requests.raise_for_status import raise_for_status

class FlowGpt(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://flowgpt.com/chat"
    working = True
    supports_gpt_35_turbo = True
    supports_message_history = True
    supports_system_message = True
    default_model = "gpt-3.5-turbo"
    models = [
        "gpt-3.5-turbo",
        "gpt-3.5-long",
        "gpt-4-turbo",
        "google-gemini",
        "claude-instant",
        "claude-v1",
        "claude-v2",
        "llama2-13b",
        "mythalion-13b",
        "pygmalion-13b",
        "chronos-hermes-13b",
        "Mixtral-8x7B",
        "Dolphin-2.6-8x7B",
    ]
    model_aliases = {
        "gemini": "google-gemini",
        "gemini-pro": "google-gemini"
    }

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        temperature: float = 0.7,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        timestamp = str(int(time.time()))
        auth = "Bearer null"
        nonce = get_random_hex()
        data = f"{timestamp}-{nonce}-{auth}"
        signature = hashlib.md5(data.encode()).hexdigest()

        headers = {
            "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:122.0) Gecko/20100101 Firefox/122.0",
            "Accept": "*/*",
            "Accept-Language": "en-US;q=0.7,en;q=0.3",
            "Accept-Encoding": "gzip, deflate, br",
            "Referer": "https://flowgpt.com/",
            "Content-Type": "application/json",
            "Authorization": "Bearer null",
            "Origin": "https://flowgpt.com",
            "Connection": "keep-alive",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-site",
            "TE": "trailers",
            "Authorization": auth,
            "x-flow-device-id": f"f-{get_random_string(19)}",
            "x-nonce": nonce,
            "x-signature": signature,
            "x-timestamp": timestamp
        }
        async with ClientSession(headers=headers) as session:
            history = [message for message in messages[:-1] if message["role"] != "system"]
            system_message = "\n".join([message["content"] for message in messages if message["role"] == "system"])
            if not system_message:
                system_message = "You are helpful assistant. Follow the user's instructions carefully."
            data = {
                "model": model,
                "nsfw": False,
                "question": messages[-1]["content"],
                "history": [{"role": "assistant", "content": "Hello, how can I help you today?"}, *history],
                "system": system_message,
                "temperature": temperature,
                "promptId": f"model-{model}",
                "documentIds": [],
                "chatFileDocumentIds": [],
                "generateImage": False,
                "generateAudio": False
            }
            async with session.post("https://prod-backend-k8s.flowgpt.com/v3/chat-anonymous", json=data, proxy=proxy) as response:
                await raise_for_status(response)
                async for chunk in response.content:
                    if chunk.strip():
                        message = json.loads(chunk)
                        if "event" not in message:
                            continue
                        if message["event"] == "text":
                            yield message["data"]



from __future__ import annotations

from ...typing import Messages
from ..base_provider import AsyncProvider, format_prompt
from ..helper import get_cookies
from ...requests import StreamSession

class Aichat(AsyncProvider):
    url = "https://chat-gpt.org/chat"
    working = False
    supports_gpt_35_turbo = True

    @staticmethod
    async def create_async(
        model: str,
        messages: Messages,
        proxy: str = None, **kwargs) -> str:
        
        cookies = get_cookies('chat-gpt.org') if not kwargs.get('cookies') else kwargs.get('cookies')
        if not cookies:
            raise RuntimeError(
                "g4f.provider.Aichat requires cookies, [refresh https://chat-gpt.org on chrome]"
            )

        headers = {
            'authority': 'chat-gpt.org',
            'accept': '*/*',
            'accept-language': 'en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3',
            'content-type': 'application/json',
            'origin': 'https://chat-gpt.org',
            'referer': 'https://chat-gpt.org/chat',
            'sec-ch-ua': '"Chromium";v="118", "Google Chrome";v="118", "Not=A?Brand";v="99"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36',
        }

        async with StreamSession(headers=headers,
                                    cookies=cookies,
                                    timeout=6,
                                    proxies={"https": proxy} if proxy else None,
                                    impersonate="chrome110", verify=False) as session:

            json_data = {
                "message": format_prompt(messages),
                "temperature": kwargs.get('temperature', 0.5),
                "presence_penalty": 0,
                "top_p": kwargs.get('top_p', 1),
                "frequency_penalty": 0,
            }

            async with session.post("https://chat-gpt.org/api/text",
                                    json=json_data) as response:

                response.raise_for_status()
                result = await response.json()

                if not result['response']:
                    raise Exception(f"Error Response: {result}")

                return result["message"]



from __future__ import annotations

import requests

from .Openai import Openai
from ...typing import AsyncResult, Messages

class OpenRouter(Openai):
    label = "OpenRouter"
    url = "https://openrouter.ai"
    working = True
    default_model = "mistralai/mistral-7b-instruct:free"

    @classmethod
    def get_models(cls):
        if not cls.models:
            url = 'https://openrouter.ai/api/v1/models'
            models = requests.get(url).json()["data"]
            cls.models = [model['id'] for model in models]
        return cls.models

    @classmethod
    def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        api_base: str = "https://openrouter.ai/api/v1",
        **kwargs
    ) -> AsyncResult:
        return super().create_async_generator(
            model, messages, api_base=api_base, **kwargs
        )


from __future__ import annotations

from aiohttp import ClientSession

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class MagickPenAsk(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://api.magickpen.com"
    api_endpoint = "/ask"
    working = True
    supports_gpt_4 = True
    default_model = "gpt-4o"

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "application/json, text/plain, */*",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/json",
            "dnt": "1",
            "origin": "https://magickpen.com",
            "priority": "u=1, i",
            "referer": "https://magickpen.com/",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-site",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            'X-API-Secret': 'WCASR6ZQJYM85DVDX7'
        }
        async with ClientSession(headers=headers) as session:
            data = {
                "query": format_prompt(messages),
                "plan": "Pay as you go"
            }
            async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy) as response:
                response.raise_for_status()
                async for chunk in response.content:
                    if chunk:
                        yield chunk.decode()



from __future__ import annotations

import re
from ..requests import StreamSession, raise_for_status
from ..typing import Messages
from .base_provider import AsyncProvider, ProviderModelMixin
from .helper import format_prompt


class Chatgpt4o(AsyncProvider, ProviderModelMixin):
    url = "https://chatgpt4o.one"
    supports_gpt_4 = True
    working = True
    _post_id = None
    _nonce = None
    default_model = 'gpt-4o'


    @classmethod
    async def create_async(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        timeout: int = 120,
        cookies: dict = None,
        **kwargs
    ) -> str:
        headers = {
            'authority': 'chatgpt4o.one',
            'accept': '*/*',
            'accept-language': 'en,fr-FR;q=0.9,fr;q=0.8,es-ES;q=0.7,es;q=0.6,en-US;q=0.5,am;q=0.4,de;q=0.3',
            'origin': 'https://chatgpt4o.one',
            'referer': 'https://chatgpt4o.one',
            'sec-ch-ua': '"Chromium";v="118", "Google Chrome";v="118", "Not=A?Brand";v="99"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36',
        }

        async with StreamSession(
            headers=headers,
            cookies=cookies,
            impersonate="chrome",
            proxies={"all": proxy},
            timeout=timeout
        ) as session:

            if not cls._post_id or not cls._nonce:
                async with session.get(f"{cls.url}/") as response:
                    await raise_for_status(response)
                    response_text = await response.text()

                    post_id_match = re.search(r'data-post-id="([0-9]+)"', response_text)
                    nonce_match = re.search(r'data-nonce="(.*?)"', response_text)

                    if not post_id_match:
                        raise RuntimeError("No post ID found")
                    cls._post_id = post_id_match.group(1)

                    if not nonce_match:
                        raise RuntimeError("No nonce found")
                    cls._nonce = nonce_match.group(1)

            prompt = format_prompt(messages)
            data = {
                "_wpnonce": cls._nonce,
                "post_id": cls._post_id,
                "url": cls.url,
                "action": "wpaicg_chat_shortcode_message",
                "message": prompt,
                "bot_id": "0"
            }

            async with session.post(f"{cls.url}/wp-admin/admin-ajax.php", data=data, cookies=cookies) as response:
                await raise_for_status(response)
                response_json = await response.json()
                if "data" not in response_json:
                    raise RuntimeError("Unexpected response structure: 'data' field missing")
                return response_json["data"]



from __future__ import annotations

import uuid

from aiohttp import ClientSession, BaseConnector

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import get_connector
from ..requests import raise_for_status

models = {
    "gpt-4o-mini-free": {
        "id": "gpt-4o-mini-free",
        "name": "GPT-4o-Mini-Free",
        "model": "ChatGPT",
        "provider": "OpenAI",
        "maxLength": 31200,
        "tokenLimit": 7800,
        "context": "8K",
    },
    "gpt-4o-mini": {
        "id": "gpt-4o-mini",
        "name": "GPT-4o-Mini",
        "model": "ChatGPT",
        "provider": "OpenAI",
        "maxLength": 260000,
        "tokenLimit": 126000,
        "context": "128K",
    },
    "gpt-4o-free": {
        "context": "8K",
        "id": "gpt-4o-free",
        "maxLength": 31200,
        "model": "ChatGPT",
        "name": "GPT-4o-free",
        "provider": "OpenAI",
        "tokenLimit": 7800,
    },
    "gpt-4-turbo-2024-04-09": {
        "id": "gpt-4-turbo-2024-04-09",
        "name": "GPT-4-Turbo",
        "model": "ChatGPT",
        "provider": "OpenAI",
        "maxLength": 260000,
        "tokenLimit": 126000,
        "context": "128K",
    },
    "gpt-4o": {
        "context": "128K",
        "id": "gpt-4o",
        "maxLength": 124000,
        "model": "ChatGPT",
        "name": "GPT-4o",
        "provider": "OpenAI",
        "tokenLimit": 62000,
    },
    "gpt-4-0613": {
        "id": "gpt-4-0613",
        "name": "GPT-4",
        "model": "ChatGPT",
        "provider": "OpenAI",
        "maxLength": 260000,
        "tokenLimit": 126000,
        "context": "128K",
    },
    "gpt-4-turbo": {
        "id": "gpt-4-turbo",
        "name": "GPT-4-Turbo",
        "model": "ChatGPT",
        "provider": "OpenAI",
        "maxLength": 260000,
        "tokenLimit": 126000,
        "context": "128K",
    },
}



class Liaobots(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://liaobots.site"
    working = True
    supports_message_history = True
    supports_system_message = True
    supports_gpt_35_turbo = True
    supports_gpt_4 = True
    default_model = "gpt-4o"
    models = list(models.keys())
    model_aliases = {
        "gpt-4o-mini": "gpt-4o-mini-free",
        "gpt-4o": "gpt-4o-free",
        "gpt-4-turbo": "gpt-4-turbo-2024-04-09",
        "gpt-4": "gpt-4-0613",
        "claude-3-opus": "claude-3-opus-20240229",
        "claude-3-sonnet": "claude-3-sonnet-20240229",
        "claude-3-5-sonnet": "claude-3-5-sonnet-20240620",
        "claude-3-haiku": "claude-3-haiku-20240307",
        "gemini-pro": "gemini-1.5-pro-latest",
        "gemini-flash": "gemini-1.5-flash-latest",
    }
    _auth_code = ""
    _cookie_jar = None

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        auth: str = None,
        proxy: str = None,
        connector: BaseConnector = None,
        **kwargs
    ) -> AsyncResult:
        model = cls.get_model(model)
        
        headers = {
            "authority": "liaobots.com",
            "content-type": "application/json",
            "origin": cls.url,
            "referer": f"{cls.url}/",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36",
        }
        async with ClientSession(
            headers=headers,
            cookie_jar=cls._cookie_jar,
            connector=get_connector(connector, proxy, True)
        ) as session:
            data = {
                "conversationId": str(uuid.uuid4()),
                "model": models[model],
                "messages": messages,
                "key": "",
                "prompt": kwargs.get("system_message", "You are a helpful assistant."),
            }
            if not cls._auth_code:
                async with session.post(
                    "https://liaobots.work/recaptcha/api/login",
                    data={"token": "abcdefghijklmnopqrst"},
                    verify_ssl=False
                ) as response:
                    await raise_for_status(response)
            try:
                async with session.post(
                    "https://liaobots.work/api/user",
                    json={"authcode": cls._auth_code},
                    verify_ssl=False
                ) as response:
                    await raise_for_status(response)
                    cls._auth_code = (await response.json(content_type=None))["authCode"]
                    if not cls._auth_code:
                        raise RuntimeError("Empty auth code")
                    cls._cookie_jar = session.cookie_jar
                async with session.post(
                    "https://liaobots.work/api/chat",
                    json=data,
                    headers={"x-auth-code": cls._auth_code},
                    verify_ssl=False
                ) as response:
                    await raise_for_status(response)
                    async for chunk in response.content.iter_any():
                        if b"<html coupert-item=" in chunk:
                            raise RuntimeError("Invalid session")
                        if chunk:
                            yield chunk.decode(errors="ignore")
            except:
                async with session.post(
                    "https://liaobots.work/api/user",
                    json={"authcode": "pTIQr4FTnVRfr"},
                    verify_ssl=False
                ) as response:
                    await raise_for_status(response)
                    cls._auth_code = (await response.json(content_type=None))["authCode"]
                    if not cls._auth_code:
                        raise RuntimeError("Empty auth code")
                    cls._cookie_jar = session.cookie_jar
                async with session.post(
                    "https://liaobots.work/api/chat",
                    json=data,
                    headers={"x-auth-code": cls._auth_code},
                    verify_ssl=False
                ) as response:
                    await raise_for_status(response)
                    async for chunk in response.content.iter_any():
                        if b"<html coupert-item=" in chunk:
                            raise RuntimeError("Invalid session")
                        if chunk:
                            yield chunk.decode(errors="ignore")

    @classmethod
    def get_model(cls, model: str) -> str:
        """
        Retrieve the internal model identifier based on the provided model name or alias.
        """
        if model in cls.model_aliases:
            model = cls.model_aliases[model]
        if model not in models:
            raise ValueError(f"Model '{model}' is not supported.")
        return model

    @classmethod
    def is_supported(cls, model: str) -> bool:
        """
        Check if the given model is supported.
        """
        return model in models or model in cls.model_aliases

    @classmethod
    async def initialize_auth_code(cls, session: ClientSession) -> None:
        """
        Initialize the auth code by making the necessary login requests.
        """
        async with session.post(
            "https://liaobots.work/api/user",
            json={"authcode": "pTIQr4FTnVRfr"},
            verify_ssl=False
        ) as response:
            await raise_for_status(response)
            cls._auth_code = (await response.json(content_type=None))["authCode"]
            if not cls._auth_code:
                raise RuntimeError("Empty auth code")
            cls._cookie_jar = session.cookie_jar

    @classmethod
    async def ensure_auth_code(cls, session: ClientSession) -> None:
        """
        Ensure the auth code is initialized, and if not, perform the initialization.
        """
        if not cls._auth_code:
            await cls.initialize_auth_code(session)



from __future__ import annotations

import re
import json
from urllib import parse
from datetime import datetime

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider
from ..requests import StreamSession


class Phind(AsyncGeneratorProvider):
    url = "https://www.phind.com"
    working = True
    supports_stream = True
    supports_message_history = True
    
    @classmethod
    async def create_async_generator(
            cls,
            model: str,
            messages: Messages,
            proxy: str = None,
            timeout: int = 120,
            creative_mode: bool = False,
            **kwargs
    ) -> AsyncResult:
        headers = {
            "Accept": "*/*",
            "Origin": cls.url,
            "Referer": f"{cls.url}/search",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
        }
        async with StreamSession(
                headers=headers,
                impersonate="chrome",
                proxies={"https": proxy},
                timeout=timeout
        ) as session:
            url = "https://www.phind.com/search?home=true"
            async with session.get(url) as response:
                text = await response.text()
                match = re.search(r'<script id="__NEXT_DATA__" type="application/json">(?P<json>[\S\s]+?)</script>',
                                  text)
                data = json.loads(match.group("json"))
                challenge_seeds = data["props"]["pageProps"]["challengeSeeds"]
            
            prompt = messages[-1]["content"]
            data = {
                "question": prompt,
                "question_history": [
                    message["content"] for message in messages[:-1] if message["role"] == "user"
                ],
                "answer_history": [
                    message["content"] for message in messages if message["role"] == "assistant"
                ],
                "webResults": [],
                "options": {
                    "date": datetime.now().strftime("%d.%m.%Y"),
                    "language": "en-US",
                    "detailed": True,
                    "anonUserId": "",
                    "answerModel": "GPT-4" if model.startswith("gpt-4") else "Phind-34B",
                    "creativeMode": creative_mode,
                    "customLinks": []
                },
                "context": "\n".join([message["content"] for message in messages if message["role"] == "system"]),
            }
            data["challenge"] = generate_challenge(data, **challenge_seeds)
            async with session.post(f"https://https.api.phind.com/infer/", headers=headers, json=data) as response:
                new_line = False
                async for line in response.iter_lines():
                    if line.startswith(b"data: "):
                        chunk = line[6:]
                        if chunk.startswith(b'<PHIND_DONE/>'):
                            break
                        if chunk.startswith(b'<PHIND_BACKEND_ERROR>'):
                            raise RuntimeError(f"Response: {chunk.decode()}")
                        if chunk.startswith(b'<PHIND_WEBRESULTS>') or chunk.startswith(b'<PHIND_FOLLOWUP>'):
                            pass
                        elif chunk.startswith(b"<PHIND_METADATA>") or chunk.startswith(b"<PHIND_INDICATOR>"):
                            pass
                        elif chunk.startswith(b"<PHIND_SPAN_BEGIN>") or chunk.startswith(b"<PHIND_SPAN_END>"):
                            pass
                        elif chunk:
                            yield chunk.decode()
                        elif new_line:
                            yield "\n"
                            new_line = False
                        else:
                            new_line = True


def deterministic_stringify(obj):
    def handle_value(value):
        if isinstance(value, (dict, list)):
            if isinstance(value, list):
                return '[' + ','.join(sorted(map(handle_value, value))) + ']'
            else:  # It's a dict
                return '{' + deterministic_stringify(value) + '}'
        elif isinstance(value, bool):
            return 'true' if value else 'false'
        elif isinstance(value, (int, float)):
            return format(value, '.8f').rstrip('0').rstrip('.')
        elif isinstance(value, str):
            return f'"{value}"'
        else:
            return 'null'
    
    items = sorted(obj.items(), key=lambda x: x[0])
    return ','.join([f'{k}:{handle_value(v)}' for k, v in items if handle_value(v) is not None])


def prng_general(seed, multiplier, addend, modulus):
    a = seed * multiplier + addend
    if a < 0:
        return ((a % modulus) - modulus) / modulus
    else:
        return a % modulus / modulus


def generate_challenge_seed(l):
    I = deterministic_stringify(l)
    d = parse.quote(I, safe='')
    return simple_hash(d)


def simple_hash(s):
    d = 0
    for char in s:
        if len(char) > 1 or ord(char) >= 256:
            continue
        d = ((d << 5) - d + ord(char[0])) & 0xFFFFFFFF
        if d > 0x7FFFFFFF:  # 2147483647
            d -= 0x100000000  # Subtract 2**32
    return d


def generate_challenge(obj, **kwargs):
    return prng_general(
        seed=generate_challenge_seed(obj),
        **kwargs
    )


from __future__ import annotations

import json
import asyncio
from aiohttp import ClientSession, TCPConnector
from urllib.parse import urlencode

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class Feedough(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://www.feedough.com"
    api_endpoint = "/wp-admin/admin-ajax.php"
    working = True
    default_model = ''

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.9",
            "content-type": "application/x-www-form-urlencoded;charset=UTF-8",
            "dnt": "1",
            "origin": cls.url,
            "referer": f"{cls.url}/ai-prompt-generator/",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Linux"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
        }

        connector = TCPConnector(ssl=False)

        async with ClientSession(headers=headers, connector=connector) as session:
            data = {
                "action": "aixg_generate",
                "prompt": format_prompt(messages),
                "aixg_generate_nonce": "110c021031"
            }

            try:
                async with session.post(
                    f"{cls.url}{cls.api_endpoint}",
                    data=urlencode(data),
                    proxy=proxy
                ) as response:
                    response.raise_for_status()
                    response_text = await response.text()
                    try:
                        response_json = json.loads(response_text)
                        if response_json.get("success") and "data" in response_json:
                            message = response_json["data"].get("message", "")
                            yield message
                    except json.JSONDecodeError:
                        yield response_text
            except Exception as e:
                print(f"An error occurred: {e}")

    @classmethod
    async def run(cls, *args, **kwargs):
        async for item in cls.create_async_generator(*args, **kwargs):
            yield item

        tasks = asyncio.all_tasks()
        for task in tasks:
            if not task.done():
                await task



from __future__ import annotations

from .types import Client, ImageProvider

from ..Provider.BingCreateImages import BingCreateImages
from ..Provider.needs_auth import Gemini, OpenaiChat
from ..Provider.You import You

class ImageModels():
    gemini = Gemini
    openai = OpenaiChat
    you = You

    def __init__(self, client: Client) -> None:
        self.client = client
        self.default = BingCreateImages(proxy=self.client.get_proxy())

    def get(self, name: str, default: ImageProvider = None) -> ImageProvider:
        return getattr(self, name) if hasattr(self, name) else default or self.default



import json
from aiohttp import ClientSession

from ..typing import Messages, AsyncResult
from .base_provider import AsyncGeneratorProvider

class Pizzagpt(AsyncGeneratorProvider):
    url = "https://www.pizzagpt.it"
    api_endpoint = "/api/chatx-completion"
    supports_message_history = False
    supports_gpt_35_turbo = True
    working = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        payload = {
            "question": messages[-1]["content"]
        }
        headers = {
            "Accept": "application/json",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Accept-Language": "en-US,en;q=0.9",
            "Content-Type": "application/json",
            "Origin": cls.url,
            "Referer": f"{cls.url}/en",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "X-Secret": "Marinara"
        }

        async with ClientSession() as session:
            async with session.post(
                f"{cls.url}{cls.api_endpoint}",
                json=payload,
                proxy=proxy,
                headers=headers
            ) as response:
                response.raise_for_status()
                response_json = await response.json()
                yield response_json["answer"]["content"]



from __future__ import annotations

from .Openai import Openai
from ...typing import AsyncResult, Messages

class PerplexityApi(Openai):
    label = "Perplexity API"
    url = "https://www.perplexity.ai"
    working = True
    default_model = "llama-3-sonar-large-32k-online"
    models = [
        "llama-3-sonar-small-32k-chat",
        "llama-3-sonar-small-32k-online",
        "llama-3-sonar-large-32k-chat",
        "llama-3-sonar-large-32k-online",
        "llama-3-8b-instruct",
        "llama-3-70b-instruct",
        "mixtral-8x7b-instruct"
    ]

    @classmethod
    def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        api_base: str = "https://api.perplexity.ai",
        **kwargs
    ) -> AsyncResult:
        return super().create_async_generator(
            model, messages, api_base=api_base, **kwargs
        )


from __future__ import annotations

import json
from aiohttp import ClientSession, ClientResponseError

from ..typing import AsyncResult, Messages
from .base_provider import AsyncGeneratorProvider, ProviderModelMixin
from .helper import format_prompt


class Marsyoo(AsyncGeneratorProvider, ProviderModelMixin):
    url = "https://aiagent.marsyoo.com"
    api_endpoint = "/api/chat-messages"
    working = True
    supports_gpt_4 = True
    default_model = 'gpt-4o'

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        headers = {
            "Accept": "*/*",
            "Accept-Language": "en-US,en;q=0.9",
            "Connection": "keep-alive",
            "DNT": "1",
            "Origin": cls.url,
            "Referer": f"{cls.url}/chat",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "authorization": "Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiI0MWNkOTE3MS1mNTg1LTRjMTktOTY0Ni01NzgxMTBjYWViNTciLCJzdWIiOiJXZWIgQVBJIFBhc3Nwb3J0IiwiYXBwX2lkIjoiNDFjZDkxNzEtZjU4NS00YzE5LTk2NDYtNTc4MTEwY2FlYjU3IiwiYXBwX2NvZGUiOiJMakhzdWJqNjhMTXZCT0JyIiwiZW5kX3VzZXJfaWQiOiI4YjE5YjY2Mi05M2E1LTRhYTktOGNjNS03MDhmNWE0YmQxNjEifQ.pOzdQ4wTrQjjRlEv1XY9TZitkW5KW1K-wbcUJAoBJ5I",
            "content-type": "application/json",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": "Linux",
        }
        async with ClientSession(headers=headers) as session:
            prompt = format_prompt(messages)
            data = {
                "response_mode": "streaming",
                "query": prompt,
                "inputs": {},
            }
            try:
                async with session.post(f"{cls.url}{cls.api_endpoint}", json=data, proxy=proxy) as response:
                    response.raise_for_status()
                    async for line in response.content:
                        if line:
                            try:
                                json_data = json.loads(line.decode('utf-8').strip().lstrip('data: '))
                                if json_data['event'] == 'message':
                                    yield json_data['answer']
                                elif json_data['event'] == 'message_end':
                                    return
                            except json.JSONDecodeError:
                                continue
            except ClientResponseError as e:
                yield f"Error: HTTP {e.status}: {e.message}"



import json
from aiohttp import ClientSession

from ..typing import Messages, AsyncResult
from .base_provider import AsyncGeneratorProvider

class Rocks(AsyncGeneratorProvider):
    url = "https://api.discord.rocks"
    api_endpoint = "/chat/completions"
    supports_message_history = False
    supports_gpt_35_turbo = True
    working = True

    @classmethod
    async def create_async_generator(
        cls,
        model: str,
        messages: Messages,
        proxy: str = None,
        **kwargs
    ) -> AsyncResult:
        payload = {"messages":messages,"model":model,"max_tokens":4096,"temperature":1,"top_p":1,"stream":True}

        headers = {
            "Accept": "application/json",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Accept-Language": "en-US,en;q=0.9",
            "Origin": cls.url,
            "Referer": f"{cls.url}/en",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
        }

        async with ClientSession() as session:
            async with session.post(
                f"{cls.url}{cls.api_endpoint}",
                json=payload,
                proxy=proxy,
                headers=headers
            ) as response:
                response.raise_for_status()
                async for line in response.content:
                    if line.startswith(b"data: "):
                        try:
                            line = json.loads(line[6:])
                        except:
                            continue
                        chunk = line["choices"][0]["delta"].get("content")
                        if chunk:
                            yield chunk
                    elif line.startswith(b"\n"):
                        pass
                    else:
                        raise Exception(f"Unexpected line: {line}")
